@@ -28,8 +28,17 @@
         ! ... potential grid, and its wave functions sub-grid.
 
         TYPE ( fft_dlay_descriptor ) :: dfftp ! descriptor for dense grid
+             !  Dimensions of the 3D real and reciprocal space FFT grid
+             !  relative to the charge density and potential ("dense" grid)
         TYPE ( fft_dlay_descriptor ) :: dffts ! descriptor for smooth grid
+             !  Dimensions of the 3D real and reciprocal space
+             !  FFT grid relative to the smooth part of the charge density
+             !  (may differ from the full charge density grid for USPP )
         TYPE ( fft_dlay_descriptor ) :: dfftb ! descriptor for box grids
+             !  Dimensions of the 3D real and reciprocal space
+             !  FFT grid relative to the "small box" computation
+             !  of the atomic augmentation part of the 
+             !  charge density used in USPP (to speed up CPV iterations)
 
         SAVE
 
@@ -37,7 +46,8 @@
 
         PUBLIC :: fft_scatter, grid_gather, grid_scatter
         PUBLIC :: dfftp, dffts, dfftb, fft_dlay_descriptor
-        PUBLIC :: cgather_sym, cgather_smooth, cscatter_sym, cscatter_smooth
+        PUBLIC :: cgather_sym, cgather_smooth, cgather_custom
+        PUBLIC :: cscatter_sym, cscatter_smooth, cscatter_custom
         PUBLIC :: gather_smooth, scatter_smooth
         PUBLIC :: tg_gather
 
@@ -55,11 +65,11 @@
 !   like infiniband, ethernet, myrinet
 !
 !-----------------------------------------------------------------------
-SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg )
+SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg )
   !-----------------------------------------------------------------------
   !
   ! transpose the fft grid across nodes
-  ! a) From columns to planes (sign > 0)
+  ! a) From columns to planes (isgn > 0)
   !
   !    "columns" (or "pencil") representation:
   !    processor "me" has ncp_(me) contiguous columns along z
@@ -76,7 +86,7 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   !    Finally one gets the "planes" representation:
   !    processor "me" has npp_(me) complete xy planes
   !
-  !  b) From planes to columns (sign < 0)
+  !  b) From planes to columns (isgn < 0)
   !
   !  Quite the same in the opposite direction
   !
@@ -85,7 +95,7 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   !  If optional argument "use_tg" is true the subroutines performs
   !  the trasposition using the Task Groups distribution
   !
-#ifdef __PARA
+#ifdef __MPI
   USE parallel_include
 #endif
   USE kinds,       ONLY : DP
@@ -94,11 +104,11 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
 
   TYPE (fft_dlay_descriptor), INTENT(in) :: dfft
 
-  INTEGER, INTENT(in)           :: nr3x, nxx_, sign, ncp_ (:), npp_ (:)
+  INTEGER, INTENT(in)           :: nr3x, nxx_, isgn, ncp_ (:), npp_ (:)
   COMPLEX (DP), INTENT(inout)   :: f_in (nxx_), f_aux (nxx_)
   LOGICAL, OPTIONAL, INTENT(in) :: use_tg
 
-#ifdef __PARA
+#ifdef __MPI
 
   INTEGER :: dest, from, k, ip, proc, ierr, me, ipoffset, nprocp, gproc, gcomm, i, kdest, kfrom
   INTEGER :: sendcount(dfft%nproc), sdispls(dfft%nproc), recvcount(dfft%nproc), rdispls(dfft%nproc)
@@ -109,6 +119,8 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   LOGICAL :: tsts(dfft%nproc), tstr(dfft%nproc)
   INTEGER :: istat( MPI_STATUS_SIZE )
 
+  INTEGER :: me_p, nppx, mc, j, npp, nnp, ii, it, ip, ioff
+
 #if defined __HPM
      !       CALL f_hpmstart( 10, 'scatter' )
 #endif
@@ -175,7 +187,7 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   !
   ierr = 0
   !
-  IF ( sign > 0 ) THEN
+  IF ( isgn > 0 ) THEN
      !
      ! "forward" scatter from columns to planes
      !
@@ -289,10 +301,139 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
         !
      ENDDO
      !
+     IF( isgn == 1 ) THEN
+
+        me_p = dfft%mype + 1
+
+        IF ( dfft%nproc == 1 ) THEN
+           nppx = dfft%nr3x
+        ELSE
+           nppx = dfft%npp( me_p )
+        ENDIF
+
+!$omp parallel default(shared)
+!$omp do
+        DO i = 1, size(f_aux)
+           f_aux(i) = (0.d0, 0.d0)
+        ENDDO
+        !
+!$omp do private(mc,j)
+        DO i = 1, dfft%nst
+           mc = dfft%ismap( i )
+           DO j = 1, dfft%npp( me_p )
+              f_aux( mc + ( j - 1 ) * dfft%nnp ) = f_in( j + ( i - 1 ) * nppx )
+           ENDDO
+        ENDDO
+!$omp end parallel
+
+     ELSE
+
+        me_p = dfft%mype + 1
+
+        IF( use_tg_ ) THEN
+           !
+           nppx = dfft%tg_npp( me_p )
+           npp  = dfft%tg_npp( me_p )
+           nnp  = dfft%nr1x * dfft%nr2x
+           !
+        ELSE
+           !
+           nppx = dfft%npp( me_p )
+           IF( dfft%nproc == 1 ) nppx = dfft%nr3x
+           npp  = dfft%npp( me_p )
+           nnp  = dfft%nnp
+           !
+        ENDIF
+        !
+!$omp parallel default(shared), private( ii, mc, j, i, ioff, ip, it )
+!$omp do
+        DO i = 1, size( f_aux )
+           f_aux(i) = (0.d0, 0.d0)
+        ENDDO
+        !
+        ii = 0
+        !
+        DO ip = 1, dfft%nproc
+           !
+           ioff = dfft%iss( ip )
+           !
+!$omp do
+           DO i = 1, dfft%nsw( ip )
+              !
+              mc = dfft%ismap( i + ioff )
+              !
+              it = ( ii + i - 1 ) * nppx
+              !
+              DO j = 1, npp
+                 f_aux( mc + ( j - 1 ) * nnp ) = f_in( j + it )
+              ENDDO
+              !
+           ENDDO
+           !
+           ii = ii + dfft%nsw( ip )
+           !
+        ENDDO
+!$omp end parallel
+
+     END IF
+     ! 
   ELSE
      !
      !  "backward" scatter from planes to columns
      !
+     IF( isgn == -1 ) THEN
+        me_p = dfft%mype + 1
+        IF ( dfft%nproc == 1 ) THEN
+           nppx = dfft%nr3x
+        ELSE
+           nppx = dfft%npp( me_p )
+        ENDIF
+!$omp parallel default(shared), private( mc, j, i )
+!$omp do
+        DO i = 1, dfft%nst
+           mc = dfft%ismap( i )
+           DO j = 1, dfft%npp( me_p )
+              f_in( j + ( i - 1 ) * nppx ) = f_aux( mc + ( j - 1 ) * dfft%nnp )
+           ENDDO
+        ENDDO
+!$omp end parallel
+
+     ELSE
+
+        me_p = dfft%mype + 1
+
+        IF( use_tg_ ) THEN
+           !
+           nppx = dfft%tg_npp( me_p )
+           npp  = dfft%tg_npp( me_p )
+           nnp  = dfft%nr1x * dfft%nr2x
+           !
+        ELSE
+           !
+           nppx = dfft%npp( me_p )
+           IF( dfft%nproc == 1 ) nppx = dfft%nr3x
+           npp  = dfft%npp( me_p )
+           nnp  = dfft%nnp
+           !
+        ENDIF
+
+!$omp parallel default(shared), private( mc, j, i, ii, ip, it )
+        ii = 0
+        DO ip = 1, dfft%nproc
+!$omp do
+           DO i = 1, dfft%nsw( ip )
+              mc = dfft%ismap( i + dfft%iss( ip ) )
+              it = (ii + i - 1)*nppx
+              DO j = 1, npp
+                 f_in( j + it ) = f_aux( mc + ( j - 1 ) * nnp )
+              ENDDO
+           ENDDO
+           ii = ii + dfft%nsw( ip )
+        ENDDO
+!$omp end parallel
+
+     END IF
+     !
      DO ip = 1, nprocp
 
         !  post the non blocking send
@@ -426,11 +567,11 @@ END SUBROUTINE fft_scatter
 !   with a defined topology, like on bluegene and cray machine
 !
 !-----------------------------------------------------------------------
-SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg )
+SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg )
   !-----------------------------------------------------------------------
   !
   ! transpose the fft grid across nodes
-  ! a) From columns to planes (sign > 0)
+  ! a) From columns to planes (isgn > 0)
   !
   !    "columns" (or "pencil") representation:
   !    processor "me" has ncp_(me) contiguous columns along z
@@ -447,7 +588,7 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   !    Finally one gets the "planes" representation:
   !    processor "me" has npp_(me) complete xy planes
   !
-  !  b) From planes to columns (sign < 0)
+  !  b) From planes to columns (isgn < 0)
   !
   !  Quite the same in the opposite direction
   !
@@ -456,7 +597,7 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   !  If optional argument "use_tg" is true the subroutines performs
   !  the trasposition using the Task Groups distribution
   !
-#ifdef __PARA
+#ifdef __MPI
   USE parallel_include
 #endif
   USE kinds,       ONLY : DP
@@ -464,14 +605,15 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   IMPLICIT NONE
 
   TYPE (fft_dlay_descriptor), INTENT(in) :: dfft
-  INTEGER, INTENT(in)           :: nr3x, nxx_, sign, ncp_ (:), npp_ (:)
+  INTEGER, INTENT(in)           :: nr3x, nxx_, isgn, ncp_ (:), npp_ (:)
   COMPLEX (DP), INTENT(inout)   :: f_in (nxx_), f_aux (nxx_)
   LOGICAL, OPTIONAL, INTENT(in) :: use_tg
 
-#ifdef __PARA
+#ifdef __MPI
 
   INTEGER :: dest, from, k, offset, proc, ierr, me, nprocp, gproc, gcomm, i, kdest, kfrom
   INTEGER :: sendcount (dfft%nproc), sdispls (dfft%nproc), recvcount (dfft%nproc), rdispls (dfft%nproc)
+  INTEGER :: me_p, nppx, mc, j, npp, nnp, ii, it, ip, ioff
   !
   LOGICAL :: use_tg_
 
@@ -495,8 +637,6 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
      nprocp = dfft%nproc
   ENDIF
   !
-  IF (nprocp.eq.1) RETURN
-  !
   CALL start_clock ('fft_scatter')
   !
   ! sendcount(proc): amount of data processor "me" must send to processor
@@ -528,7 +668,9 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
   !
 
   ierr = 0
-  IF (sign.gt.0) THEN
+  IF (isgn.gt.0) THEN
+
+     IF( nprocp < 2 ) GO TO 10 
      !
      ! "forward" scatter from columns to planes
      !
@@ -545,28 +687,16 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
            gproc = proc
         ENDIF
         !
-        !  optimize for large parallel execution, where npp_ ( gproc ) ~ 1
-        !
-        IF( npp_ ( gproc ) > 128 ) THEN
-           DO k = 1, ncp_ (me)
-              CALL DCOPY (2 * npp_ ( gproc ), f_in (from + (k - 1) * nr3x), &
-                   1, f_aux (dest + (k - 1) * npp_ ( gproc ) ), 1)
+        DO k = 1, ncp_ (me)
+           kdest = dest + (k - 1) * npp_ ( gproc ) - 1
+           kfrom = from + (k - 1) * nr3x - 1
+           DO i = 1, npp_ ( gproc )
+              f_aux ( kdest + i ) =  f_in ( kfrom + i )
            ENDDO
-        ELSEIF( npp_ ( gproc ) == 1 ) THEN
-           DO k = 1, ncp_ (me)
-              f_aux (dest + (k - 1) ) =  f_in (from + (k - 1) * nr3x )
-           ENDDO
-        ELSE
-           DO k = 1, ncp_ (me)
-              kdest = dest + (k - 1) * npp_ ( gproc ) - 1
-              kfrom = from + (k - 1) * nr3x - 1
-              DO i = 1, npp_ ( gproc )
-                 f_aux ( kdest + i ) =  f_in ( kfrom + i )
-              ENDDO
-           ENDDO
-        ENDIF
+        ENDDO
         offset = offset + npp_ ( gproc )
      ENDDO
+
      !
      ! maybe useless; ensures that no garbage is present in the output
      !
@@ -587,10 +717,144 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
 
      IF( abs(ierr) /= 0 ) CALL errore ('fft_scatter', 'info<>0', abs(ierr) )
      !
+10   CONTINUE
+
+     IF( isgn == 1 ) THEN
+
+        me_p = dfft%mype + 1
+
+        IF ( dfft%nproc == 1 ) THEN
+           nppx = dfft%nr3x
+        ELSE
+           nppx = dfft%npp( me_p )
+        ENDIF
+
+!$omp parallel default(shared)
+!$omp do
+        DO i = 1, size(f_aux)
+           f_aux(i) = (0.d0, 0.d0)
+        ENDDO
+        !
+!$omp do private(mc,j)
+        DO i = 1, dfft%nst
+           mc = dfft%ismap( i )
+           DO j = 1, dfft%npp( me_p )
+              f_aux( mc + ( j - 1 ) * dfft%nnp ) = f_in( j + ( i - 1 ) * nppx )
+           ENDDO
+        ENDDO
+!$omp end parallel
+
+     ELSE
+
+        me_p = dfft%mype + 1
+
+        IF( use_tg_ ) THEN
+           !
+           nppx = dfft%tg_npp( me_p )
+           npp  = dfft%tg_npp( me_p )
+           nnp  = dfft%nr1x * dfft%nr2x
+           !
+        ELSE
+           !
+           nppx = dfft%npp( me_p )
+           IF( dfft%nproc == 1 ) nppx = dfft%nr3x
+           npp  = dfft%npp( me_p )
+           nnp  = dfft%nnp
+           !
+        ENDIF
+        !
+!$omp parallel default(shared), private( ii, mc, j, i, ioff, ip, it )
+!$omp do
+        DO i = 1, size( f_aux )
+           f_aux(i) = (0.d0, 0.d0)
+        ENDDO
+        !
+        ii = 0
+        !
+        DO ip = 1, dfft%nproc
+           !
+           ioff = dfft%iss( ip )
+           !
+!$omp do
+           DO i = 1, dfft%nsw( ip )
+              !
+              mc = dfft%ismap( i + ioff )
+              !
+              it = ( ii + i - 1 ) * nppx
+              !
+              DO j = 1, npp
+                 f_aux( mc + ( j - 1 ) * nnp ) = f_in( j + it )
+              ENDDO
+              !
+           ENDDO
+           !
+           ii = ii + dfft%nsw( ip )
+           !
+        ENDDO
+!$omp end parallel
+
+
+     END IF
+
   ELSE
      !
      !  "backward" scatter from planes to columns
      !
+     IF( isgn == -1 ) THEN
+        me_p = dfft%mype + 1
+        IF ( dfft%nproc == 1 ) THEN
+           nppx = dfft%nr3x
+        ELSE
+           nppx = dfft%npp( me_p )
+        ENDIF
+!$omp parallel default(shared), private( mc, j, i )
+!$omp do
+        DO i = 1, dfft%nst
+           mc = dfft%ismap( i )
+           DO j = 1, dfft%npp( me_p )
+              f_in( j + ( i - 1 ) * nppx ) = f_aux( mc + ( j - 1 ) * dfft%nnp )
+           ENDDO
+        ENDDO
+!$omp end parallel
+
+     ELSE
+
+        me_p = dfft%mype + 1
+
+        IF( use_tg_ ) THEN
+           !
+           nppx = dfft%tg_npp( me_p )
+           npp  = dfft%tg_npp( me_p )
+           nnp  = dfft%nr1x * dfft%nr2x
+           !
+        ELSE
+           !
+           nppx = dfft%npp( me_p )
+           IF( dfft%nproc == 1 ) nppx = dfft%nr3x
+           npp  = dfft%npp( me_p )
+           nnp  = dfft%nnp
+           !
+        ENDIF
+
+!$omp parallel default(shared), private( mc, j, i, ii, ip, it )
+        ii = 0
+        DO ip = 1, dfft%nproc
+!$omp do
+           DO i = 1, dfft%nsw( ip )
+              mc = dfft%ismap( i + dfft%iss( ip ) )
+              it = (ii + i - 1)*nppx
+              DO j = 1, npp
+                 f_in( j + it ) = f_aux( mc + ( j - 1 ) * nnp )
+              ENDDO
+           ENDDO
+           ii = ii + dfft%nsw( ip )
+        ENDDO
+!$omp end parallel
+
+     END IF
+
+     IF( nprocp < 2 ) GO TO 20
+     !
      !  step two: communication
      !
      IF( use_tg_ ) THEN
@@ -621,31 +885,20 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, sign, use_tg
            gproc = proc
         ENDIF
         !
-        !  optimize for large parallel execution, where npp_ ( gproc ) ~ 1
-        !
-        IF( npp_ ( gproc ) > 128 ) THEN
-           DO k = 1, ncp_ (me)
-              CALL DCOPY ( 2 * npp_ ( gproc ), f_aux (from + (k - 1) * npp_ ( gproc ) ), 1, &
-                                            f_in  (dest + (k - 1) * nr3x ), 1 )
+        DO k = 1, ncp_ (me)
+           kdest = dest + (k - 1) * nr3x - 1
+           kfrom = from + (k - 1) * npp_ ( gproc ) - 1
+           DO i = 1, npp_ ( gproc )
+              f_in ( kdest + i ) = f_aux( kfrom + i )
            ENDDO
-        ELSEIF ( npp_ ( gproc ) == 1 ) THEN
-           DO k = 1, ncp_ (me)
-              f_in ( dest + (k - 1) * nr3x ) = f_aux ( from + (k - 1) )
-           ENDDO
-        ELSE
-           DO k = 1, ncp_ (me)
-              kdest = dest + (k - 1) * nr3x - 1
-              kfrom = from + (k - 1) * npp_ ( gproc ) - 1
-              DO i = 1, npp_ ( gproc )
-                 f_in ( kdest + i ) = f_aux( kfrom + i )
-              ENDDO
-           ENDDO
-        ENDIF
+        ENDDO
         !
         offset = offset + npp_ ( gproc )
         !
      ENDDO
 
+20   CONTINUE
+
   ENDIF
 
   CALL stop_clock ('fft_scatter')
@@ -678,7 +931,7 @@ SUBROUTINE grid_gather( f_in, f_out )
   !
   REAL(DP) :: f_in( : ), f_out( : )
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
@@ -742,7 +995,7 @@ SUBROUTINE grid_scatter( f_in, f_out )
   !
   REAL(DP) :: f_in( : ), f_out( : )
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
@@ -808,7 +1061,7 @@ SUBROUTINE cgather_sym( f_in, f_out )
   !
   COMPLEX(DP) :: f_in( : ), f_out(:)
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
@@ -869,7 +1122,7 @@ SUBROUTINE cgather_smooth ( f_in, f_out )
   !
   COMPLEX(DP) :: f_in(:), f_out(:)
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
@@ -909,6 +1162,66 @@ SUBROUTINE cgather_smooth ( f_in, f_out )
   !
 END SUBROUTINE cgather_smooth
 !
+!----------------------------------------------------------------------------
+SUBROUTINE cgather_custom ( f_in, f_out, dfftt )
+  !----------------------------------------------------------------------------
+  !
+  ! ... gathers data on the custom AND complex fft grid
+  !
+  ! ... gathers nproc distributed data on the first processor of every pool
+  !
+  ! ... COMPLEX*16  f_in  = distributed variable ( dfftt%nnr )
+  ! ... COMPLEX*16  f_out = gathered variable (nr1sx*nr2sx*nr3sx)
+  !
+  USE mp,        ONLY : mp_barrier
+  USE kinds,     ONLY : DP
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP) :: f_in(:), f_out(:)
+  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfftt 
+  !
+#if defined (__MPI)
+  !
+  INTEGER :: proc, info
+  INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
+  !
+  !
+  CALL start_clock( 'gather' )
+  !
+  DO proc = 0, ( dfftp%nproc - 1 )
+     !
+     recvcount(proc) = 2 * dfftt%nnp * dfftt%npp(proc+1)
+     !
+     IF ( proc == 0 ) THEN
+        !
+        displs(proc) = 0
+        !
+     ELSE
+        !
+        displs(proc) = displs(proc-1) + recvcount(proc-1)
+        !
+     ENDIF
+     !
+  ENDDO
+  !
+  CALL mp_barrier( dfftp%comm )
+  !
+  CALL MPI_GATHERV( f_in, recvcount(dfftp%mype), MPI_DOUBLE_PRECISION, f_out, &
+                    recvcount, displs, MPI_DOUBLE_PRECISION, dfftp%root,    &
+                    dfftp%comm, info )
+  !
+  CALL errore( 'gather', 'info<>0', info )
+  !
+  CALL stop_clock( 'gather' )
+  !
+#endif
+  !
+  RETURN
+  !
+END SUBROUTINE cgather_custom
+!
 ! ... "scatter"-like subroutines
 !
 !----------------------------------------------------------------------------
@@ -928,7 +1241,7 @@ SUBROUTINE cscatter_sym( f_in, f_out )
   !
   COMPLEX(DP) :: f_in(:), f_out(:)
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
@@ -988,7 +1301,7 @@ SUBROUTINE cscatter_smooth( f_in, f_out )
   !
   COMPLEX(DP) :: f_in(:), f_out(:)
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
@@ -1029,9 +1342,68 @@ SUBROUTINE cscatter_smooth( f_in, f_out )
   RETURN
   !
 END SUBROUTINE cscatter_smooth
-
-
-
+!
+!----------------------------------------------------------------------------
+SUBROUTINE cscatter_custom( f_in, f_out, dfftt )
+  !----------------------------------------------------------------------------
+  !
+  ! ... scatters data on the custom AND complex fft grid
+  ! ... scatters data from the first processor of every pool
+  !
+  ! ... COMPLEX*16  f_in  = gathered variable (nr1sx*nr2sx*nr3sx)
+  ! ... COMPLEX*16  f_out = distributed variable ( dfftt%nnr)
+  !
+  USE mp,        ONLY : mp_barrier
+  USE kinds,     ONLY : DP
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP) :: f_in(:), f_out(:)
+  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfftt 
+  !
+#if defined (__MPI)
+  !
+  INTEGER :: proc, info
+  INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
+  !
+  !
+  CALL start_clock( 'scatter' )
+  !
+  DO proc = 0, ( dfftp%nproc - 1 )
+     !
+     sendcount(proc) = 2 * dfftt%nnp * dfftt%npp(proc+1)
+     !
+     IF ( proc == 0 ) THEN
+        !
+        displs(proc) = 0
+        !
+     ELSE
+        !
+        displs(proc) = displs(proc-1) + sendcount(proc-1)
+        !
+     ENDIF
+     !
+  ENDDO
+  !
+  CALL mp_barrier( dfftp%comm )
+  !
+  CALL MPI_SCATTERV( f_in, sendcount, displs, MPI_DOUBLE_PRECISION,   &
+                     f_out, sendcount(dfftp%mype), MPI_DOUBLE_PRECISION, &
+                     dfftp%root, dfftp%comm, info )
+  !
+  CALL errore( 'scatter', 'info<>0', info )
+  !
+  IF ( sendcount(dfftp%mype) /=  dfftt%nnr  ) f_out(sendcount(dfftp%mype)+1: dfftt%nnr ) = 0.D0
+  !
+  CALL stop_clock( 'scatter' )
+  !
+#endif
+  !
+  RETURN
+  !
+END SUBROUTINE cscatter_custom
+!
 !----------------------------------------------------------------------------
 SUBROUTINE gather_smooth ( f_in, f_out )
   !----------------------------------------------------------------------------
@@ -1051,7 +1423,7 @@ SUBROUTINE gather_smooth ( f_in, f_out )
   !
   REAL(DP) :: f_in(:), f_out(:)
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dffts%nproc-1), recvcount(0:dffts%nproc-1)
@@ -1109,7 +1481,7 @@ SUBROUTINE scatter_smooth( f_in, f_out )
   !
   REAL(DP) :: f_in(:), f_out(:)
   !
-#if defined (__PARA)
+#if defined (__MPI)
   !
   INTEGER :: proc, info
   INTEGER :: displs(0:dffts%nproc-1), sendcount(0:dffts%nproc-1)
@@ -1200,7 +1572,7 @@ SUBROUTINE tg_gather( dffts, v, tg_v )
       tg_v( i ) = 0.0d0
    ENDDO
 
-#if defined (__PARA) && defined (__MPI)
+#if defined (__MPI)
 
    CALL MPI_Allgatherv( v(1), nsiz, MPI_DOUBLE_PRECISION, &
         tg_v(1), recv_cnt, recv_displ, MPI_DOUBLE_PRECISION, dffts%ogrp_comm, IERR)
