@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2008 Quantum ESPRESSO group
+! Copyright (C) 2001-2012 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -7,12 +7,12 @@
 !
 !--------------------------------------------------------------------------!
 ! FFT scalar drivers Module - contains machine-dependent routines for:     !
-! FFTW, FFTW3, ESSL, LINUX_ESSL, SCSL, SUNPERF, NEC ASL and ACML libraries !
+! FFTW, FFTW3, ESSL, LINUX_ESSL, SCSL, SUNPERF, NEC ASL libraries          !
 ! (both 3d for serial execution and 1d+2d FFTs for parallel execution,     !
 ! excepted NEC ASL, 3d only, no parallel execution)                        !
 ! Written by Carlo Cavazzoni, modified by P. Giannozzi, contributions      !
 ! by Martin Hilgemans, Guido Roma, Pascal Thibaudeau, Stephane Lefranc,    !
-! Nicolas Lacorne, Filippo Spiga - Last update Aug 2008                    !
+! Nicolas Lacorne, Filippo Spiga - Last update Aug 2012                    !
 !--------------------------------------------------------------------------!
 
 #include "fft_defs.h"
@@ -138,14 +138,6 @@
      C_POINTER, SAVE :: fw_planz( ndims ) = 0
      C_POINTER, SAVE :: bw_planz( ndims ) = 0
 
-#elif defined __ACML
-
-     INTEGER, PARAMETER :: ltabl = 3 * nfftx + 100
-     INTEGER :: INFO
-
-     COMPLEX (DP), SAVE :: fw_tablez( ltabl, ndims )
-     COMPLEX (DP), SAVE :: bw_tablez( ltabl, ndims )
-
 #elif defined __ESSL || defined __LINUX_ESSL
 
      !   ESSL IBM library: see the ESSL manual for DCFT
@@ -219,14 +211,6 @@
        idir = -1; CALL CREATE_PLAN_1D( fw_planz( icurrent), nz, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_planz( icurrent), nz, idir)
 
-#elif defined __ACML
-
-      tscale = 1.0_DP / nz
-      CALL ZFFT1MX(0, tscale, .FALSE., nsl, nz, c(1), 1, ldz, &
-            cout(1), 1, ldz, fw_tablez(1, icurrent), INFO)
-      CALL ZFFT1MX(0, 1.0_DP, .FALSE., nsl, nz, c(1), 1, ldz, &
-            cout(1), 1, ldz, bw_tablez(1, icurrent), INFO)
-
 #elif defined __FFTW3
 
 
@@ -332,17 +316,6 @@
 
 #endif
 
-#elif defined __ACML
-
-     IF( isign < 0 ) THEN
-        tscale = 1.0_DP / nz
-        CALL ZFFT1MX(-1, tscale, .FALSE., nsl, ldz, c(1), 1, ldz, cout(1), 1, ldz, &
-                fw_tablez(1, ip),INFO)
-     ELSE IF( isign > 0 ) THEN
-        CALL ZFFT1MX(1, 1.0_DP, .FALSE., nsl, ldz, c(1), 1, ldz, cout(1), 1, ldz, &
-                bw_tablez(1, ip),INFO)
-     END IF
-
 #elif defined __FFTW3
 
      IF (isign < 0) THEN
@@ -476,14 +449,6 @@
      C_POINTER, SAVE :: fw_plan( 2, ndims ) = 0
      C_POINTER, SAVE :: bw_plan( 2, ndims ) = 0
 
-#elif defined __ACML
-
-     INTEGER, PARAMETER :: ltabl = 3 * nfftx + 100
-     INTEGER :: INFO
-
-     COMPLEX (DP), SAVE :: fw_tablex( ltabl, ndims ), bw_tablex( ltabl, ndims )
-     COMPLEX (DP), SAVE :: fw_tabley( ltabl, ndims ), bw_tabley( ltabl, ndims )
-
 #elif defined __ESSL || defined __LINUX_ESSL
 
      INTEGER, PARAMETER :: ltabl = 20000 + 3 * nfftx
@@ -567,22 +532,6 @@
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 1,icurrent), nx, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_plan( 1,icurrent), nx, idir)
 
-#elif  defined __ACML
-
-      tscale = 1.0_DP / ( nx * ny )
-
-#if defined __OPENMP
-      CALL ZFFT1MX(0, 1.0_DP, .TRUE., nx, ny, r(1), ldx, 1, r(1), ldx, 1,fw_tabley(1, icurrent), INFO)
-      CALL ZFFT1MX(0, 1.0_DP, .TRUE., nx, ny, r(1), ldx, 1, r(1), ldx, 1, bw_tabley(1, icurrent), INFO)
-      CALL ZFFT1MX(0, tscale, .TRUE., ny, nx, r(1), 1, ldx, r(1), 1, ldx, fw_tablex(1, icurrent), INFO)
-      CALL ZFFT1MX(0, 1.0_DP, .TRUE., ny, nx, r(1), 1, ldx, r(1), 1, ldx, bw_tablex(1, icurrent), INFO)
-#else
-      CALL ZFFT1MX(0, 1.0_DP, .TRUE., 1, ny, r(1), ldx, 1, r(1), ldx, 1,fw_tabley(1, icurrent), INFO)
-      CALL ZFFT1MX(0, 1.0_DP, .TRUE., 1, ny, r(1), ldx, 1, r(1), ldx, 1, bw_tabley(1, icurrent), INFO)
-      CALL ZFFT1MX(0, tscale, .TRUE., ny, nx, r(1), 1, ldx, r(1), 1, ldx, fw_tablex(1, icurrent), INFO)
-      CALL ZFFT1MX(0, 1.0_DP, .TRUE., ny, nx, r(1), 1, ldx, r(1), 1, ldx, bw_tablex(1, icurrent), INFO)
-#endif
-
 #elif defined __FFTW3
 
        IF ( ldx /= nx .OR. ldy /= ny ) THEN
@@ -809,56 +758,6 @@
 
 #endif
 
-#elif defined __ACML
-
-#if defined __OPENMP
-
-   IF( isign < 0 ) THEN
-      tscale = 1.0_DP / ( nx*ny )
-      do k = 1, nzl
-         kk = 1 + ( k - 1 ) * ldx * ldy
-         CALL ZFFT1MX(-1, tscale, .TRUE., ny, nx, r(kk), 1, ldx, r(kk), 1, ldx, fw_tablex(1, ip), INFO)
-         CALL ZFFT1MX(-1, 1.0_DP, .TRUE., nx, ny, r(kk), ldx, 1, r(kk), ldx, 1, fw_tabley(1, ip), INFO)
-      end do
-   ELSE IF( isign > 0 ) THEN
-      DO k = 1, nzl
-         kk = 1 + ( k - 1 ) * ldx * ldy
-         CALL ZFFT1MX(1, 1.0_DP, .TRUE., nx, ny, r(kk), ldx, 1, r(kk), ldx, 1, bw_tabley(1, ip), INFO)
-         CALL ZFFT1MX(1, 1.0_DP, .TRUE., ny, nx, r(kk), 1, ldx, r(kk), 1, ldx, bw_tablex(1, ip), INFO)
-      END DO
-   END IF
-
-
-#else
-
-   IF( isign < 0 ) THEN
-      tscale = 1.0_DP / ( nx * ny )
-      do k = 1, nzl
-         kk = 1 + ( k - 1 ) * ldx * ldy
-         CALL ZFFT1MX(-1, tscale, .TRUE., ny, nx, r(kk), 1, ldx, r(kk), 1, ldx, fw_tablex(1, ip), INFO)
-         do i = 1, nx
-            IF( dofft( i ) ) THEN
-               kk = i + ( k - 1 ) * ldx * ldy
-               CALL ZFFT1MX(-1, 1.0_DP, .TRUE., 1, ny, r(kk), ldx, 1, r(kk), ldx, 1,fw_tabley(1, ip), INFO)
-            END IF
-         end do
-      end do
-   ELSE IF( isign > 0 ) THEN
-      DO k = 1, nzl
-         do i = 1, nx
-            IF( dofft( i ) ) THEN
-               kk = i + ( k - 1 ) * ldx * ldy
-               CALL ZFFT1MX(1, 1.0_DP, .TRUE., 1, ny, r(kk), ldx, 1, r(kk), ldx, 1, bw_tabley(1, ip), INFO)
-            END IF
-         end do
-         kk = 1 + ( k - 1 ) * ldx * ldy
-         CALL ZFFT1MX(1, 1.0_DP, .TRUE., ny, nx, r(kk), 1, ldx, r(kk), 1, ldx, bw_tablex(1, ip), INFO)
-      END DO
-
-   END IF
-
-#endif
-
 #elif defined __FFTW3
 
      IF ( ldx /= nx .OR. ldy /= ny ) THEN
@@ -1165,15 +1064,6 @@
      C_POINTER, save :: fw_plan(ndims) = 0
      C_POINTER, save :: bw_plan(ndims) = 0
 
-#elif defined __ACML
-
-     INTEGER, PARAMETER :: ltabl = 4 * nfftx + 300
-     INTEGER :: INFO
-
-     COMPLEX (DP), SAVE :: fw_table(ltabl,ndims)
-     COMPLEX (DP), SAVE :: bw_table(ltabl,ndims)
-
-
 #elif defined __SCSL
 
      INTEGER, PARAMETER :: ltabl = (2 * nfftx + 256)*3
@@ -1251,16 +1141,6 @@
        idir = -1; CALL CREATE_PLAN_3D( fw_plan(icurrent), nx, ny, nz, idir)
        idir =  1; CALL CREATE_PLAN_3D( bw_plan(icurrent), nx, ny, nz, idir)
 
-#elif defined __ACML
-
-       IF ( nx /= ldx .or. ny /= ldy .or. nz /= ldz ) &
-         call errore('cfft3','not implemented',1)
-
-
-       tscale = 1.0_DP / DBLE( nx * ny * nz )
-       CALL ZFFT3DY (0,tscale,.TRUE., nx,ny,nz,f(1),1,ldx,ldx*ldy,f(1),1,ldx, ldx*ldy, fw_table(1, icurrent),ltabl,INFO)
-       CALL ZFFT3DY (0,1.0_DP,.TRUE.,  nx,ny,nz,f(1),1,ldx,ldx*ldy,f(1),1,ldx, ldx*ldy, bw_table(1, icurrent),ltabl,INFO)
-
 #elif defined __FFTW3
 
        IF ( nx /= ldx .or. ny /= ldy .or. nz /= ldz ) &
@@ -1332,15 +1212,6 @@
        call FFTW_INPLACE_DRV_3D( bw_plan(ip), 1, f(1), 1, 1 )
      END IF
 
-#elif defined __ACML
-
-     IF( isign < 0 ) THEN
-         tscale = 1.0_DP / DBLE( nx * ny * nz )
-         CALL ZFFT3DY (-1,tscale,.TRUE., nx,ny,nz,f(1),1,ldx,ldx*ldy,f(1),1,ldx,ldx*ldy,fw_table(1, ip),ltabl,INFO)
-     ELSE IF( isign > 0 ) THEN
-         CALL ZFFT3DY (1,1.0_DP,.TRUE.,  nx,ny,nz,f(1),1,ldx,ldx*ldy,f(1),1,ldx,ldx*ldy,bw_table(1, ip),ltabl,INFO)
-     END IF
-
 #elif defined __FFTW3
 
    IF( isign < 0 ) THEN
@@ -1481,14 +1352,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
   C_POINTER, SAVE :: fw_plan ( 3, ndims ) = 0
   C_POINTER, SAVE :: bw_plan ( 3, ndims ) = 0
 
-#elif defined __ACML
-
-  INTEGER, PARAMETER :: ltabl = 3 * nfftx + 100
-  INTEGER :: INFO
-
-  COMPLEX (DP), SAVE :: fw_table( ltabl, 3, ndims )
-  COMPLEX (DP), SAVE :: bw_table( ltabl, 3, ndims )
-
 #elif defined __ESSL || defined __LINUX_ESSL
 
   INTEGER, PARAMETER :: ltabl = 20000 + 3 * nfftx
@@ -1546,26 +1409,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 3, icurrent), nz, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_plan( 3, icurrent), nz, idir)
 
-#elif defined __ACML
-
-       !  x - direction
-       incx1 = 1; incx2 = ldx; m = 1
-
-       CALL ZFFT1MX(0, 1.0_DP, .TRUE., m, nx, f(1), incx1, incx2, f(1), incx1, incx2, fw_table(1, 1, icurrent), INFO)
-       CALL ZFFT1MX(0, 1.0_DP, .TRUE., m, nx, f(1), incx1, incx2, f(1), incx1, incx2, bw_table(1, 1, icurrent), INFO)
-
-       !  y - direction
-       incx1 = ldx; incx2 = 1; m = nx;
-
-       CALL ZFFT1MX(0, 1.0_DP, .TRUE., m, ny, f(1), incx1, incx2, f(1), incx1, incx2, fw_table(1, 2, icurrent), INFO)
-       CALL ZFFT1MX(0, 1.0_DP, .TRUE., m, ny, f(1), incx1, incx2, f(1), incx1, incx2, bw_table(1, 2, icurrent), INFO)
-
-       !  z - direction
-       incx1 = ldx * ldy; incx2 = 1; m = ldx * ny
-
-       CALL ZFFT1MX(0, 1.0_DP, .TRUE., m, nz, f(1), incx1, incx2, f(1), incx1, incx2, fw_table(1, 3, icurrent), INFO)
-       CALL ZFFT1MX(0, 1.0_DP, .TRUE., m, nz, f(1), incx1, incx2, f(1), incx1, incx2, bw_table(1, 3, icurrent), INFO)
-
 #elif defined __FFTW3
        IF( fw_plan( 1, icurrent) /= 0 ) &
             CALL dfftw_destroy_plan( fw_plan( 1, icurrent) )
@@ -1656,15 +1499,13 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
               if ( do_fft_x( jj ) == 1 ) THEN
 #if defined __FFTW
                 call FFTW_INPLACE_DRV_1D( bw_plan( 1, ip), m, f( ii ), incx1, incx2 )
-#elif defined __ACML
-                CALL ZFFT1MX(1, 1.0_DP, .TRUE., m, nx, f(ii), incx1, incx2, f(ii), incx1, incx2, bw_table(1, 1, ip), INFO)
 #elif defined __FFTW3
                 call dfftw_execute_dft( bw_plan( 1, ip), f( ii: ), f( ii: ) )
 #elif defined __ESSL || defined __LINUX_ESSL
-                call dcft (0, f ( ii ), incx1, incx2, f ( ii ), incx1, incx2, nx, m, &
-                  -isign, 1.0_DP, bw_table ( 1, 1,  ip ), ltabl, work( 1 ), lwork)
+                call dcft (0, f (ii), incx1,incx2, f (ii), incx1,incx2, nx, m, &
+                -isign, 1.0_DP, bw_table ( 1, 1,  ip ), ltabl, work( 1 ), lwork)
 #else
-                call errore(' cfft3ds ',' no scalar fft driver specified ', 1)
+                call errore(' cfft3ds ',' no scalar fft driver specified ', 2)
 #endif
               endif
            enddo
@@ -1681,15 +1522,13 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
            if ( do_fft_y( k ) == 1 ) then
 #if defined __FFTW
              call FFTW_INPLACE_DRV_1D( bw_plan( 2, ip), m, f( ii ), incx1, incx2 )
-#elif defined __ACML
-             CALL ZFFT1MX(1, 1.0_DP, .TRUE., m, ny, f(ii), incx1, incx2, f(ii), incx1, incx2, bw_table(1, 2, ip), INFO)
 #elif defined __FFTW3
              call dfftw_execute_dft( bw_plan( 2, ip), f( ii: ), f( ii: ) )
 #elif defined __ESSL || defined __LINUX_ESSL
-             call dcft (0, f ( ii ), incx1, incx2, f ( ii ), incx1, incx2, ny, m, &
+             call dcft (0, f (ii), incx1, incx2, f (ii), incx1, incx2, nx, m, &
                -isign, 1.0_DP, bw_table ( 1, 2,  ip ), ltabl, work( 1 ), lwork)
 #else
-             call errore(' cfft3ds ',' no scalar fft driver specified ', 1)
+             call errore(' cfft3ds ',' no scalar fft driver specified ', 3)
 #endif
            endif
         enddo
@@ -1702,8 +1541,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
 
 #if defined __FFTW
         call FFTW_INPLACE_DRV_1D( bw_plan( 3, ip), m, f( 1 ), incx1, incx2 )
-#elif defined __ACML
-        CALL ZFFT1MX(1, 1.0_DP, .TRUE., m, nz, f(1), incx1, incx2, f(1), incx1, incx2, bw_table(1, 3, ip), INFO)
 #elif defined __FFTW3
         call dfftw_execute_dft( bw_plan( 3, ip), f(1:), f(1:) )
 #elif defined __ESSL || defined __LINUX_ESSL
@@ -1721,8 +1558,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
 
 #if defined __FFTW
         call FFTW_INPLACE_DRV_1D( fw_plan( 3, ip), m, f( 1 ), incx1, incx2 )
-#elif defined __ACML
-       CALL ZFFT1MX(-1, 1.0_DP, .TRUE., m, nz, f(1), incx1, incx2, f(1), incx1, incx2, fw_table(1, 3, ip), INFO)
 #elif defined __FFTW3
         call dfftw_execute_dft( fw_plan( 3, ip), f(1:), f(1:) )
 #elif defined __ESSL || defined __LINUX_ESSL
@@ -1742,15 +1577,13 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
            if ( do_fft_y ( k ) == 1 ) then
 #if defined __FFTW
              call FFTW_INPLACE_DRV_1D( fw_plan( 2, ip), m, f( ii ), incx1, incx2 )
-#elif defined __ACML
-             CALL ZFFT1MX(-1, 1.0_DP, .TRUE., m, ny, f(ii), incx1, incx2, f(ii), incx1, incx2, fw_table(1, 2, ip), INFO)
 #elif defined __FFTW3
              call dfftw_execute_dft( fw_plan( 2, ip), f( ii: ), f( ii: ) )
 #elif defined __ESSL || defined __LINUX_ESSL
-             call dcft (0, f ( ii ), incx1, incx2, f ( ii ), incx1, incx2, ny, m, &
+             call dcft (0, f (ii), incx1, incx2, f (ii), incx1, incx2, ny, m, &
                -isign, 1.0_DP, fw_table ( 1, 2, ip ), ltabl, work( 1 ), lwork)
 #else
-             call errore(' cfft3ds ',' no scalar fft driver specified ', 1)
+             call errore(' cfft3ds ',' no scalar fft driver specified ', 4)
 #endif
            endif
         enddo
@@ -1768,15 +1601,13 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
               if ( do_fft_x( jj ) == 1 ) then
 #if defined __FFTW
                 call FFTW_INPLACE_DRV_1D( fw_plan( 1, ip), m, f( ii ), incx1, incx2 )
-#elif defined __ACML
-                CALL ZFFT1MX(-1, 1.0_DP, .TRUE., m, nx, f(ii), incx1, incx2, f(ii), incx1, incx2, fw_table(1, 1, ip), INFO)
 #elif defined __FFTW3
                 call dfftw_execute_dft( fw_plan( 1, ip), f( ii: ), f( ii: ) )
 #elif defined __ESSL || defined __LINUX_ESSL
-                call dcft (0, f ( ii ), incx1, incx2, f ( ii ), incx1, incx2, nx, m, &
-                   -isign, 1.0_DP, fw_table ( 1, 1, ip ), ltabl, work( 1 ), lwork)
+                call dcft (0, f (ii), incx1,incx2, f (ii), incx1,incx2, nx, m, &
+                 -isign, 1.0_DP, fw_table ( 1, 1, ip ), ltabl, work( 1 ), lwork)
 #else
-                call errore(' cfft3ds ',' no scalar fft driver specified ', 1)
+                call errore(' cfft3ds ',' no scalar fft driver specified ', 5)
 #endif
               endif
            enddo
@@ -1826,15 +1657,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
       C_POINTER, save :: bw_plany(  ndims ) = 0
       C_POINTER, save :: bw_planxy( ndims ) = 0
 
-#elif defined __ACML
-
-     INTEGER, PARAMETER :: ltabl = 3 * nfftx + 100
-     INTEGER :: INFO
-
-      COMPLEX (DP), save :: aux3( ltabl, ndims )
-      COMPLEX (DP), save :: aux2( ltabl, ndims )
-      COMPLEX (DP), save :: aux1( ltabl, ndims )
-
 #elif defined __ESSL || defined __LINUX_ESSL
 
       INTEGER, PARAMETER :: ltabl = 20000 + 3 * nfftx
@@ -1907,19 +1729,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
              call DESTROY_PLAN_2D( bw_planxy(icurrent) )
         call CREATE_PLAN_2D( bw_planxy(icurrent), nx, ny, 1 )
 !
-#elif defined __ACML
-
-         if( nz /= dims(3,icurrent) ) then
-            CALL ZFFT1MX(0, 1.0_DP, .TRUE., ldx*ldy, nz, f(1), ldx*ldy, 1, f(1), ldx*ldy, 1,  aux3(1, icurrent), INFO)
-         end if
-
-         CALL ZFFT1MX(0, 1.0_DP, .TRUE., ldy*nplanes, nx, f(1), 1, ldx, f(1), 1, ldx,  aux1(1, icurrent), INFO)
-
-         if( ny /= dims(2,icurrent) ) then
-              CALL ZFFT1MX(0, 1.0_DP, .TRUE., ldx, ny,  f(1), ldx, 1, f(1), ldx, 1,  aux2(1, icurrent), INFO)
-         end if
-
-
 #elif defined __FFTW3
 
         if ( bw_planz(icurrent) /= 0 ) &
@@ -1985,23 +1794,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
         call FFTW_INPLACE_DRV_1D( bw_planx(ip), ny, f((k-1)*ldx*ldy + 1), 1, ldx )
       end do   
 
-#elif defined __ACML
-
-      !   fft in the z-direction...
-
-      CALL ZFFT1MX(1, 1.0_DP, .TRUE., ldx*ldy, nz, f(1), ldx*ldy, 1, f(1), ldx*ldy, 1, aux3(1, ip), INFO)
-
-      !   x-direction
-
-      CALL ZFFT1MX(1, 1.0_DP, .TRUE., ldy*nplanes, nx, f(nstart), 1, ldx, f(nstart), 1, ldx, aux1(1, ip), INFO)
-
-      !   y-direction
-
-      DO K = imin3, imax3
-        nstart = ( k - 1 ) * ldx * ldy + 1
-        CALL ZFFT1MX(1, 1.0_DP, .TRUE., ldx, ny,  f(nstart), ldx, 1, f(nstart), ldx, 1, aux2(1, ip), INFO)
-      END DO
-
 #elif defined __FFTW3
 
       call dfftw_execute_dft(bw_planz(ip), f(1:), f(1:))
