@@ -40,6 +40,7 @@ module funct
 !
   USE io_global, ONLY: stdout
   USE kinds,     ONLY: DP
+  USE control_flags,  ONLY : lwfpbe0, lwfpbe0nscf ! Lingzhu Kong
   IMPLICIT NONE
   PRIVATE
   SAVE
@@ -82,7 +83,7 @@ module funct
   ! one of the following keywords ("dft_shortname"):
   !              "pz"    = "sla+pz"            = Perdew-Zunger LDA
   !              "bp"    = "b88+p86"           = Becke-Perdew grad.corr.
-  !              "pw91"  = "pw +ggx+ggc"       = PW91 (aka GGA)
+  !              "pw91"  = "sla+pw+ggx+ggc"    = PW91 (aka GGA)
   !              "blyp"  = "sla+b88+lyp+blyp"  = BLYP
   !              "pbe"   = "sla+pw+pbx+pbc"    = PBE
   !              "revpbe"= "sla+pw+rpb+pbc"    = revPBE (Zhang-Yang)
@@ -92,7 +93,7 @@ module funct
   !              "wc"    = "sla+pw+wcx+pbc"    = Wu-Cohen
   !              "tpss"  = "sla+pw+tpss+tpss"  = TPSS Meta-GGA
   !              "pbe0"  = "pb0x+pw+pb0x+pbc"  = PBE0
-  !              "hse"   = "nox+pw+hse+pbc"    = Heyd-Scuseria-Ernzerhof HSE 06
+  !              "hse"   = "sla+pw+hse+pbc"    = Heyd-Scuseria-Ernzerhof HSE 06
   !              "b3lyp" = "b3lp+vwn+b3lp+b3lp"= B3LYP
   !              "vdw-df"= "sla+pw+rpb+vdw1"   = vdW-DF
   !              "vdw-df2"="sla+pw+rpb+vdw2"   = vdW-DF2
@@ -189,6 +190,8 @@ module funct
   !              pbesol  J.P. Perdew et al., PRL 100, 136406 (2008)
   !              vdW-DF  M. Dion et al., PRL 92, 246401 (2004)
   !                      T. Thonhauser et al., PRB 76, 125112 (2007)
+  !              vdw-DF2 Lee et al., Phys. Rev. B 82, 081101 (2010)
+  !              c09x    V. R. Cooper, Phys. Rev. B 81, 161104(R) (2010)
   !
   integer, parameter:: notset = -1
   !
@@ -285,7 +288,6 @@ CONTAINS
     do l = 1, len
        dftout (l:l) = capital (dft_(l:l) )
     enddo
-
     !
     ! ----------------------------------------------
     ! FIRST WE CHECK ALL THE SPECIAL NAMES
@@ -293,7 +295,6 @@ CONTAINS
     !       not using function "matches"
     ! ----------------------------------------------
     !
-
     if ( 'REVPBE' .EQ. TRIM(dftout) ) then
     ! special case : revPBE
        call set_dft_value (iexch,1) !Default
@@ -402,7 +403,7 @@ CONTAINS
     ! special case : PBC  = PW + PBC 
        call set_dft_value (iexch,1) !Default
        call set_dft_value (icorr,4)
-       call set_dft_value (igcx,0) !Default       
+       call set_dft_value (igcx, 0) !Default       
        call set_dft_value (igcc, 4)
        call set_dft_value (inlc,0) !Default    
        dft_defined = .true.
@@ -419,36 +420,27 @@ CONTAINS
     ! special case : PW91 = GGX + GGC
     else if ('PW91'.EQ. TRIM(dftout) ) then
        call set_dft_value (iexch,1) !Default
-       call set_dft_value (icorr,1) !Default
+       call set_dft_value (icorr,4) 
        call set_dft_value (igcx, 2)
        call set_dft_value (igcc, 2)
        call set_dft_value (inlc,0) !Default    
        dft_defined = .true.
        
-    ! special case : HCTH already contains LDA exchange and correlation
+    ! special case : HCTH 
     else if ('HCTH'.EQ. TRIM(dftout)) then
-       call set_dft_value(iexch,0)
-       call set_dft_value(icorr,0)
-       call set_dft_value (igcx, 0) !Default    
-       call set_dft_value (igcc, 0) !Default    
-       call set_dft_value (inlc,0) !Default    
-       dft_defined = .true.
-              
-    ! special case : OPTX already contains LDA exchange
-    else if ('OPTX'.EQ. TRIM(dftout)) then
-       call set_dft_value(iexch,0)
-       call set_dft_value(icorr,0) !Default    
-       call set_dft_value (igcx, 0) !Default    
-       call set_dft_value (igcc, 0) !Default    
+       call set_dft_value(iexch,0) ! contained in hcth
+       call set_dft_value(icorr,0) ! contained in hcth
+       call set_dft_value (igcx,5)
+       call set_dft_value (igcc,5)
        call set_dft_value (inlc,0) !Default    
        dft_defined = .true.
               
     ! special case : OLYP = OPTX + LYP
     else if ('OLYP'.EQ. TRIM(dftout)) then
-       call set_dft_value(iexch,0)
+       call set_dft_value(iexch,0) ! contained in optx
        call set_dft_value(icorr,3)
-       call set_dft_value(igcx,6)
-       call set_dft_value(igcc,3)
+       call set_dft_value(igcx, 6)
+       call set_dft_value(igcc, 3)
        call set_dft_value (inlc,0) !Default    
        dft_defined = .true.
        
@@ -693,7 +685,7 @@ CONTAINS
     !character(len=*), intent(in) :: dft_
     !logical, intent(in), optional :: nomsg
 
-    iexch = 1; icorr = 0; igcx = 0; igcc = 0
+    iexch = 0; icorr = 0; igcx = 0; igcc = 0
     exx_fraction = 1.0_DP
     ishybrid = ( exx_fraction /= 0.0_DP )
 
@@ -991,6 +983,13 @@ subroutine xc (rho, ex, ec, vx, vc)
   !..exchange
   if (iexch == 1) THEN             !  'sla'
      call slater (rs, ex, vx)
+!==================================================================
+!Lingzhu Kong
+     IF(lwfpbe0 .or. lwfpbe0nscf)THEN
+         ex = 0.75d0 * ex
+         vx = 0.75d0 * vx
+     END IF
+!==================================================================
   ELSEIF (iexch == 2) THEN         !  'sl1'
      call slater1(rs, ex, vx)
   ELSEIF (iexch == 3) THEN         !  'rxc'
@@ -1089,6 +1088,14 @@ subroutine xc_spin (rho, zeta, ex, ec, vxup, vxdw, vcup, vcdw)
   !..exchange
   IF (iexch == 1) THEN      ! 'sla'
      call slater_spin (rho, zeta, ex, vxup, vxdw)
+!==============================================================
+!Lingzhu Kong
+     IF( lwfpbe0 .or. lwfpbe0nscf )THEN
+          ex   = 0.75d0 * ex
+          vxup = 0.75d0 * vxup
+          vxdw = 0.75d0 * vxdw
+     END IF
+!==============================================================
   ELSEIF (iexch == 2) THEN  ! 'sl1'
      call slater1_spin (rho, zeta, ex, vxup, vxdw)
   ELSEIF (iexch == 3) THEN  ! 'rxc'
@@ -1169,6 +1176,12 @@ subroutine xc_spin_vec (rho, zeta, length, evx, evc)
   select case (iexch)
   case(1)            ! 'sla'
      call slater_spin_vec (rho, zeta, evx, length)
+!=========================================================
+!Lingzhu Kong
+     if (lwfpbe0 .or. lwfpbe0nscf) then
+        evx = 0.75_DP * evx
+     end if
+!=========================================================
   case(2)            ! 'sl1'
      do i=1,length
         call slater1_spin (rho(i), zeta(i), evx(i,3), evx(i,1), evx(i,2))
@@ -1272,6 +1285,14 @@ subroutine gcxc (rho, grho, sx, sc, v1x, v2x, v1c, v2c)
      call ggax (rho, grho, sx, v1x, v2x)
   elseif (igcx == 3) then
      call pbex (rho, grho, 1, sx, v1x, v2x)
+!==============================================================
+!Lingzhu Kong
+     IF( lwfpbe0 .or. lwfpbe0nscf )THEN
+          sx  = 0.75d0 * sx
+          v1x = 0.75d0 * v1x
+          v2x = 0.75d0 * v2x
+     END IF
+!==============================================================
   elseif (igcx == 4) then
      call pbex (rho, grho, 2, sx, v1x, v2x)
   elseif (igcx == 5 .and. igcc == 5) then
@@ -1450,6 +1471,16 @@ subroutine gcx_spin (rhoup, rhodw, grhoup2, grhodw2, &
        v2xup = (1.0_DP - exx_fraction) * v2xup
        v2xdw = (1.0_DP - exx_fraction) * v2xdw
      end if
+!=============================================================
+!Lingzhu Kong
+     if (igcx == 3 .and. (lwfpbe0 .or. lwfpbe0nscf) ) then
+       sx =    0.75D0 * sx
+       v1xup = 0.75D0 * v1xup
+       v1xdw = 0.75D0 * v1xdw
+       v2xup = 0.75D0 * v2xup
+       v2xdw = 0.75D0 * v2xdw
+     end if
+!=============================================================
      if (igcx == 12 .and. exx_started ) then
 
         call pbexsr_lsd (rhoup, rhodw, grhoup2, grhodw2, sxsr,  &
@@ -1614,6 +1645,16 @@ subroutine gcx_spin_vec(rhoup, rhodw, grhoup2, grhodw2, &
         v2xup = (1.0_DP - exx_fraction) * v2xup
         v2xdw = (1.0_DP - exx_fraction) * v2xdw
      end if
+!=============================================================
+!Lingzhu Kong
+     if (igcx == 3 .and. (lwfpbe0 .or. lwfpbe0nscf) ) then
+       sx =    0.75D0 * sx
+       v1xup = 0.75D0 * v1xup
+       v1xdw = 0.75D0 * v1xdw
+       v2xup = 0.75D0 * v2xup
+       v2xdw = 0.75D0 * v2xdw
+     end if
+!=============================================================
   case(9)
      do i=1,length
         if (rhoup(i) > small .and. sqrt(abs(grhoup2(i)) ) > small) then
@@ -1663,7 +1704,7 @@ subroutine gcx_spin_vec(rhoup, rhodw, grhoup2, grhodw2, &
      v2xdw = 2.0_DP * v2xdw
 
   case default
-     call errore ('gcx_spin', 'not implemented', igcx)
+     call errore ('gcx_spin_vec', 'not implemented', igcx)
   end select
   !
   if (igcx.ne.0) then
