@@ -26,7 +26,7 @@ MODULE input_parameters
 !=----------------------------------------------------------------------------=!
   !
   USE kinds,      ONLY : DP
-  USE parameters, ONLY : nsx, npk, lqmax
+  USE parameters, ONLY : nsx, lqmax
   USE wannier_new,ONLY : wannier_data
   !
   IMPLICIT NONE
@@ -96,15 +96,15 @@ MODULE input_parameters
         CHARACTER(len=80) :: calculation = 'none'
           ! Specify the type of the simulation
           ! See below for allowed values
-        CHARACTER(len=80) :: calculation_allowed(12)
+        CHARACTER(len=80) :: calculation_allowed(15)
         DATA calculation_allowed / 'scf', 'nscf', 'relax', 'md', 'cp', &
-          'vc-relax', 'vc-md', 'vc-cp', 'bands', 'neb', 'smd', 'cp-wf'/
-
+          'vc-relax', 'vc-md', 'vc-cp', 'bands', 'neb', 'smd', 'cp-wf', &
+          'cp-wf-nscf','cp-wf-pbe0', 'pbe0-nscf'/   ! Lingzhu Kong
         CHARACTER(len=80) :: verbosity = 'default'
           ! define the verbosity of the code output
         CHARACTER(len=80) :: verbosity_allowed(6)
-        DATA verbosity_allowed / 'high' , 'medium', 'default' , 'low' , 'minimal' ,&
-                                 'default+projwfc' /
+        DATA verbosity_allowed / 'debug', 'high', 'medium', 'default', &
+                                 'low', 'minimal' /
 
         CHARACTER(len=80) :: restart_mode = 'restart'
           ! specify how to start/restart the simulation
@@ -352,6 +352,9 @@ MODULE input_parameters
           ! if .true. use symmetry only to symmetrize k points
         LOGICAL :: force_symmorphic = .false.
           ! if .true. disable fractionary translations (nonsymmorphic groups)
+        LOGICAL :: use_all_frac = .false.
+          ! if .true. enable usage of all fractionary translations, 
+          ! disabling check if they are commensurate with FFT grid
 
         REAL(DP) :: ecfixed = 0.0_DP, qcutz = 0.0_DP, q2sigma = 0.0_DP
           ! parameters for modified kinetic energy functional to be used
@@ -367,9 +370,11 @@ MODULE input_parameters
 
         LOGICAL :: lda_plus_u = .false.
           ! Use DFT+U method - following are the needed parameters
+        INTEGER :: lda_plus_u_kind = 0
         INTEGER, PARAMETER :: nspinx=2
         REAL(DP) :: starting_ns_eigenvalue(lqmax,nspinx,nsx) = -1.0_DP
         REAL(DP) :: hubbard_u(nsx) = 0.0_DP
+        REAL(DP) :: hubbard_j(3,nsx) = 0.0_DP
         REAL(DP) :: hubbard_alpha(nsx) = 0.0_DP
         CHARACTER(len=80) :: U_projection_type = 'atomic'
 
@@ -381,21 +386,19 @@ MODULE input_parameters
         REAL(DP) :: sigma_pen(10) = 0.01_DP
         REAL(DP) :: alpha_pen(10) = 0.0_DP
 
-#if defined (EXX)
-          ! All the variables here are PWSCF ONLY
+          ! next group of variables PWSCF ONLY
           !
         INTEGER  :: nqx1 = 1, nqx2 = 1, nqx3=1
           !
         REAL(DP) :: exx_fraction = -1.0_DP      ! if negative, use defaults
         REAL(DP) :: screening_parameter = -1.0_DP
           !
-
         CHARACTER(len=80) :: exxdiv_treatment = 'gygi-baldereschi'
           ! define how ro cure the Coulomb divergence in EXX
           ! Allowed values are:
-        CHARACTER(len=80) :: exxdiv_treatment_allowed(8)
-        DATA exxdiv_treatment_allowed / 'gygi-baldereschi', 'gygi-bald', 'g-b', &
-             'yukawa', 'vcut_ws', 'vcut_spherical', 'erfc_simple', 'none' /
+        CHARACTER(len=80) :: exxdiv_treatment_allowed(6)
+        DATA exxdiv_treatment_allowed / 'gygi-baldereschi', 'gygi-bald', 'g-b',&
+             'vcut_ws', 'vcut_spherical', 'none' /
           !
         LOGICAL  :: x_gamma_extrapolation = .true.
         REAL(DP) :: yukawa = 0.0_DP
@@ -404,14 +407,15 @@ MODULE input_parameters
         LOGICAL  :: adaptive_thr = .FALSE.
         REAL(DP) :: conv_thr_init = 0.001_DP
         REAL(DP) :: conv_thr_multi = 0.1_DP
-#endif
+        REAL(DP) :: ecutfock = -1.d0
 
+          ! parameters for external electric field
         INTEGER  :: edir = 0
         REAL(DP) :: emaxpos = 0.0_DP
         REAL(DP) :: eopreg = 0.0_DP
         REAL(DP) :: eamp = 0.0_DP
-          ! parameters for external electroc field
 
+          ! Various parameters for noncollinear calculations
         LOGICAL  :: noncolin = .false.
         LOGICAL  :: lspinorb = .false.
         LOGICAL  :: starting_spin_angle=.FALSE.
@@ -420,22 +424,19 @@ MODULE input_parameters
         REAL(DP) :: angle1(nsx) = 0.0_DP
         REAL(DP) :: angle2(nsx) = 0.0_DP
         INTEGER  :: report = 1
-          ! Various parameters for noncollinear calculationso
         LOGICAL  :: no_t_rev = .FALSE.
 
         CHARACTER(len=80) :: constrained_magnetization = 'none'
-          ! Used to perform constrained calculations in magnetic systems
         REAL(DP) :: B_field(3) = 0.0_DP
           ! A fixed magnetic field defined by the vector B_field is added
           ! to the exchange and correlation magnetic field.
 
         CHARACTER(len=80) :: sic = 'none'
           ! CP only - SIC correction (D'avezac Mauri)
-
+          ! Parameters for SIC calculation
         REAL(DP) :: sic_epsilon = 0.0_DP
         REAL(DP) :: sic_alpha   = 0.0_DP
         LOGICAL   :: force_pairing = .false.
-          ! Parameters for SIC calculation
 
         LOGICAL :: spline_ps = .false.
           ! use spline interpolation for pseudopotential
@@ -451,9 +452,9 @@ MODULE input_parameters
           ! other DFT-D parameters ( see PW/mm_dispersion.f90 )
         REAL ( DP ) :: london_s6   =   0.75_DP , & ! default global scaling parameter for PBE
                        london_rcut = 200.00_DP
-#ifdef __SOLVENT
+#ifdef __ENVIRON
 !
-        LOGICAL   :: do_solvent = .false.
+        LOGICAL   :: do_environ = .false.
 #endif
 !
         CHARACTER(LEN=3) :: esm_bc = 'pbc'
@@ -480,20 +481,19 @@ MODULE input_parameters
           ! for abs(gp)<=esm_debug_gpmax (gp is integer and has tpiba unit)
 
         NAMELIST / system / ibrav, celldm, a, b, c, cosab, cosac, cosbc, nat, &
-             ntyp, nbnd, ecutwfc, ecutrho, nr1, nr2, nr3, nr1s, nr2s,  &
-             nr3s, nr1b, nr2b, nr3b, nosym, nosym_evc, noinv,                 &
+             ntyp, nbnd, ecutwfc, ecutrho, nr1, nr2, nr3, nr1s, nr2s,         &
+             nr3s, nr1b, nr2b, nr3b, nosym, nosym_evc, noinv, use_all_frac,   &
              force_symmorphic, starting_magnetization,                        &
-             occupations, degauss, nspin, ecfixed,              &
-             qcutz, q2sigma, lda_plus_U, Hubbard_U, Hubbard_alpha,            &
+             occupations, degauss, nspin, ecfixed,                            &
+             qcutz, q2sigma, lda_plus_U, lda_plus_u_kind,                     &
+             Hubbard_U, Hubbard_J, Hubbard_alpha,                             &
              edir, emaxpos, eopreg, eamp, smearing, starting_ns_eigenvalue,   &
              U_projection_type, input_dft, la2F, assume_isolated,             &
-#if defined (EXX)
-             x_gamma_extrapolation, nqx1, nqx2, nqx3,                         &
+             nqx1, nqx2, nqx3, ecutfock,                                      &
              exxdiv_treatment, x_gamma_extrapolation, yukawa, ecutvcut,       &
              exx_fraction, screening_parameter,                               &
-#endif
-#ifdef __SOLVENT
-             do_solvent,                                                      &
+#ifdef __ENVIRON
+             do_environ,                                                      &
 #endif
              noncolin, lspinorb, starting_spin_angle, lambda, angle1, angle2, &
              report,              &
@@ -503,10 +503,10 @@ MODULE input_parameters
              spline_ps, one_atom_occupations, london, london_s6, london_rcut, &
              step_pen, A_pen, sigma_pen, alpha_pen, no_t_rev,                 &
              esm_bc, esm_efield, esm_w, esm_nfit, esm_debug, esm_debug_gpmax
-#ifdef __SOLVENT
+#ifdef __ENVIRON
 !
 !=----------------------------------------------------------------------------=!
-!  SOLVENT Namelist Input Parameters
+!  ENVIRON Namelist Input Parameters
 !=----------------------------------------------------------------------------=!
 !
 ! Global parameters
@@ -515,8 +515,14 @@ MODULE input_parameters
         ! verbosity  0: only prints summary of polarization charge calculation; 
         !    1: prints an extra file with details of iterative convergence;
         !    2: prints 3D cube files of physical properties
-        REAL(DP) :: solvent_thr = 1.d-1
+        REAL(DP) :: environ_thr = 1.d-1
         ! how early in scf should the corrective pot start being calculated
+        CHARACTER( LEN = 256 ) :: environ_type = 'input'
+        ! keyword to set up all the environment parameters at once to a specific set
+        ! vacuum = all the flags are off (perm=1.d0, surf=0.0, pres=0.0)
+        ! water = parameters optimized for water solutions in Andreussi et al. 
+        !         J. Chem. Phys. 136, 064102 (perm=78, surf=50, pres=-0.35)
+        ! input = do not use any predefined set, use paramters from input
 !
 ! Switching function parameters
 !
@@ -524,7 +530,7 @@ MODULE input_parameters
         ! type of switching functions used in the solvation models
         !    0: original Fattebert-Gygi
         !    1: ultrasoft dielectric function as defined in Andreussi et al.
-        REAL(DP) :: rhozero = 0.001
+        REAL(DP) :: rhomax = 0.001
         ! first parameter of the sw function, roughly corresponding 
         ! to the density threshold of the solvation model
         REAL(DP) :: rhomin = 0.0001
@@ -534,8 +540,8 @@ MODULE input_parameters
 !
 ! Dielectric solvent parameters
 !
-        REAL(DP) :: epsinfty = 78.D0
-        ! epsinfty dielectric permittivity of the solvation model. If set equal
+        REAL(DP) :: env_static_permittivity = 1.D0
+        ! static dielectric permittivity of the solvation model. If set equal
         ! to one (=vacuum) no dielectric effects
         CHARACTER( LEN = 256 ) :: eps_mode = 'electronic'
         !  eps_mode method for calculating the density that sets 
@@ -553,33 +559,65 @@ MODULE input_parameters
         ! ionic dielectric function is adopted
         REAL(DP) :: atomicspread(nsx) = 0.5D0
         ! gaussian spreads of the atomic density of charge
+        LOGICAL :: add_jellium = .false.
+        ! depending on periodic boundary corrections, one may need to explicitly
+        ! polarize the compensatinig jellium background
+!
+! Numerical differentiators paramters
+!
+        INTEGER  :: ifdtype = 1 
+        ! type of numerical differentiator: 1=central differences, 
+        ! 2=low-noise lanczos (m=2), 3=low-noise lanczos (m=4), 
+        ! 4=smooth noise-robust (n=2), 5=smooth noise-robust (n=4)
+        INTEGER  :: nfdpoint = 1
+        ! number of points used in the numerical differentiator 
+        ! N = 2*nfdpoint+1
 !
 ! Iterative solver parameters
 !
+        CHARACTER( LEN=256 ) :: mixtype = 'linear'
+        ! mixing method for iterative calculation of polarization charges
+        ! 'linear', 'anderson', 'diis', 'broyden'
         REAL(DP) :: mixrhopol = 0.5D0
         ! mixing param to be used in iter calculation of polarization charges
         REAL(DP) :: tolrhopol = 1.D-10
         ! convergence threshold for polarization charges in iterative procedure
+        INTEGER :: ndiis=1
+        ! order of DIIS interpolation of iterative polarization charge
 !
 ! Cavitation energy parameters
 !
-        REAL(DP) :: gamma = 0.D0
+        REAL(DP) :: env_surface_tension = 0.D0
         ! solvent surface tension, if equal to zero no cavitation term 
         REAL(DP) :: delta = 0.00001D0
         ! finite difference parameter to compute the molecular surface
 !
 ! PV energy parameters
 !
-        REAL(DP) :: extpressure = 0.D0
+        REAL(DP) :: env_pressure = 0.D0
         ! external pressure for PV energy, if equal to zero no pressure term 
+!
+! Ionic countercharge parameters
+!
+        REAL(DP) :: cion = 0.D0
+        ! molar concentration of ionic countercharge (M=mol/L)
+        REAL(DP) :: zion = 1.D0
+        ! valence of ionic countercharge
+        REAL(DP) :: rhopb = 0.0001D0
+        ! density threshold for the onset of ionic countercharge
+        REAL(DP) :: solvent_temperature = 300.D0
+        ! temperature of the solution
 
-        NAMELIST / solvent /                                           &
-             verbose, solvent_thr,                                     &
-             stype, rhozero, rhomin, tbeta,                            &
-             epsinfty, eps_mode, solvationrad, atomicspread,           &
-             mixrhopol, tolrhopol,                                     &
-             gamma, delta,                                             &
-             extpressure
+        NAMELIST / environ /                                           &
+             verbose, environ_thr, environ_type,                       &
+             stype, rhomax, rhomin, tbeta,                             &
+             env_static_permittivity, eps_mode,                        &
+             solvationrad, atomicspread, add_jellium,                  &
+             ifdtype, nfdpoint,                                        &
+             mixtype, ndiis, mixrhopol, tolrhopol,                     &
+             env_surface_tension, delta,                               &
+             env_pressure,                                             &
+             cion, zion, rhopb, solvent_temperature
 #endif
 !
 !=----------------------------------------------------------------------------=!
@@ -939,9 +977,7 @@ MODULE input_parameters
           mixing_mode, mixing_beta, mixing_ndim, mixing_fixed_ns,      &
           tqr, diago_cg_maxiter, diago_david_ndim, diagonalization ,   &
           startingpot, startingwfc , conv_thr,                         &
-#if defined (EXX)
           adaptive_thr, conv_thr_init, conv_thr_multi,                 &
-#endif
           diago_thr_init, n_inner, fermi_energy, rotmass, occmass,     &
           rotation_damping, occupation_damping, rotation_dynamics,     &
           occupation_dynamics, tcg, maxiter, etresh, passop, epol,     &
@@ -1273,7 +1309,16 @@ MODULE input_parameters
           REAL(DP) :: maxwfdt
           REAL(DP) :: wf_q
           REAL(DP) :: wf_friction
-          !
+!=======================================================================
+!Lingzhu Kong
+          INTEGER  :: vnbsp
+          INTEGER  :: neigh
+          REAL(DP) :: poisson_eps
+          REAL(DP) :: dis_cutoff
+          REAL(DP) :: exx_ps_rcut
+          REAL(DP) :: exx_me_rcut
+!=======================================================================
+
           INTEGER :: nit
           INTEGER :: nsd
           INTEGER :: nsteps
@@ -1287,12 +1332,14 @@ MODULE input_parameters
           INTEGER :: wffort
           !
           LOGICAL :: writev
-          !
-          NAMELIST / wannier / wf_efield, wf_switch, sw_len, efx0, efy0, efz0, &
-                               efx1, efy1, efz1, wfsd, wfdt, maxwfdt, wf_q,    &
-                               wf_friction, nit, nsd, nsteps, tolw, adapt,     &
-                               calwf, nwf, wffort, writev
-
+!==============================================================================
+!Lingzhu Kong
+          NAMELIST / wannier / wf_efield, wf_switch, sw_len, efx0, efy0, efz0,&
+                               efx1, efy1, efz1, wfsd, wfdt, neigh,poisson_eps,&
+                               dis_cutoff,exx_ps_rcut, exx_me_rcut, vnbsp,    &
+                               maxwfdt, wf_q, wf_friction, nit, nsd, nsteps,  & 
+                               tolw, adapt, calwf, nwf, wffort, writev
+!===============================================================================
 !  END manual
 ! ----------------------------------------------------------------------
 
@@ -1342,13 +1389,11 @@ MODULE input_parameters
         LOGICAL   :: tocc = .false.
         LOGICAL   :: tcell = .false.
         LOGICAL   :: tdipole = .false.
-        LOGICAL   :: tsetnfi = .false.
         LOGICAL   :: tionvel = .false.
         LOGICAL   :: tconstr = .false.
         LOGICAL   :: tesr = .false.
         LOGICAL   :: tksout = .false.
         LOGICAL   :: ttemplate = .false.
-        LOGICAL   :: tvhmean = .false.
         LOGICAL   :: twannier = .false.
 
 !
@@ -1384,7 +1429,7 @@ MODULE input_parameters
 !
 ! ...   k-points inputs
         LOGICAL :: tk_inp = .false.
-        REAL(DP) :: xk(3,npk) = 0.0_DP, wk(npk) = 0.0_DP
+        REAL(DP), ALLOCATABLE :: xk(:,:), wk(:)
         INTEGER :: nkstot = 0, nk1 = 0, nk2 = 0, nk3 = 0, k1 = 0, k2 = 0, k3 = 0
         CHARACTER(len=80) :: k_points = 'gamma'
           ! k_points = 'automatic' | 'crystal' | 'tpiba' | 'gamma'*
@@ -1399,31 +1444,11 @@ MODULE input_parameters
           !  number that gives the number of points between the present point
           !  and the next. The weight of the last point is not used.
 !
-!    NEWNFI
-!
-        LOGICAL :: tnewnfi_card = .false.
-        INTEGER :: newnfi_card = 0
-
-!
-!    2DPROCMESH
-!
-        LOGICAL :: t2dpegrid_inp = .false.
-
-!
 !    OCCUPATIONS
 !
         REAL(DP), ALLOCATABLE :: f_inp(:,:)
         LOGICAL   :: tf_inp = .false.
 
-!
-!    VHMEAN
-!
-! ...   card planar mean of the Hartree potential
-        LOGICAL :: tvhmean_inp = .false.
-        INTEGER :: vhnr_inp = 0, vhiunit_inp = 0
-        REAL(DP)  :: vhrmin_inp = 0.0_dP, vhrmax_inp = 0.0_DP
-        CHARACTER :: vhasse_inp = 'X'
-
 !
 !    DIPOLE
 !
@@ -1438,7 +1463,6 @@ MODULE input_parameters
 !    CELL_PARAMETERS
 !
        REAL(DP) :: rd_ht(3,3) = 0.0_DP
-       CHARACTER(len=80) :: cell_symmetry = 'none'
        CHARACTER(len=80) :: cell_units = 'alat'
        LOGICAL   :: trd_ht = .false.
 
@@ -1456,17 +1480,6 @@ MODULE input_parameters
       REAL(DP),          ALLOCATABLE :: constr_target_inp(:)
       LOGICAL,           ALLOCATABLE :: constr_target_set(:)
 
-!
-!    COLLECTIVE_VARS
-!
-      INTEGER  :: ncolvar_inp    = 0
-      REAL(DP) :: colvar_tol_inp = 1.E-6_DP
-      !
-      CHARACTER(len=20), ALLOCATABLE :: colvar_type_inp(:)
-      REAL(DP),          ALLOCATABLE :: colvar_inp(:,:)
-      REAL(DP),          ALLOCATABLE :: colvar_target_inp(:)
-      LOGICAL,           ALLOCATABLE :: colvar_target_set(:)
-
 !
 !    KOHN_SHAM
 !
@@ -1560,28 +1573,6 @@ CONTAINS
     !
   END SUBROUTINE allocate_input_constr
 
-  SUBROUTINE allocate_input_colvar()
-    !
-    IF ( allocated( colvar_type_inp ) )   DEALLOCATE( colvar_type_inp )
-    IF ( allocated( colvar_inp ) )        DEALLOCATE( colvar_inp )
-    IF ( allocated( colvar_target_inp ) ) DEALLOCATE( colvar_target_inp )
-    IF ( allocated( colvar_target_set ) ) DEALLOCATE( colvar_target_set )
-    !
-    ALLOCATE( colvar_type_inp(   ncolvar_inp ) )
-    ALLOCATE( colvar_target_inp(     ncolvar_inp ) )
-    ALLOCATE( colvar_target_set( ncolvar_inp ) )
-    !
-    ALLOCATE( colvar_inp( nc_fields, ncolvar_inp ) )
-    !
-    colvar_type_inp   = ' '
-    colvar_inp        = 0.0_DP
-    colvar_target_inp     = 0.0_DP
-    colvar_target_set = .false.
-    !
-    RETURN
-    !
-  END SUBROUTINE allocate_input_colvar
-  !
   SUBROUTINE allocate_input_iprnks( nksx, nspin )
     !
     INTEGER, INTENT(in) :: nksx, nspin
@@ -1599,6 +1590,8 @@ CONTAINS
 
   SUBROUTINE deallocate_input_parameters()
     !
+    IF ( allocated( xk ) ) DEALLOCATE( xk )
+    IF ( allocated( wk ) ) DEALLOCATE( wk )
     IF ( allocated( rd_pos ) ) DEALLOCATE( rd_pos )
     IF ( allocated( sp_pos ) ) DEALLOCATE( sp_pos )
     IF ( allocated( if_pos ) ) DEALLOCATE( if_pos )
@@ -1614,11 +1607,6 @@ CONTAINS
     IF ( allocated( constr_target_inp ) ) DEALLOCATE( constr_target_inp )
     IF ( allocated( constr_target_set ) ) DEALLOCATE( constr_target_set )
     !
-    IF ( allocated( colvar_type_inp ) )   DEALLOCATE( colvar_type_inp )
-    IF ( allocated( colvar_inp ) )        DEALLOCATE( colvar_inp )
-    IF ( allocated( colvar_target_inp ) ) DEALLOCATE( colvar_target_inp )
-    IF ( allocated( colvar_target_set ) ) DEALLOCATE( colvar_target_set )
-    !
     IF ( allocated( iprnks ) )       DEALLOCATE( iprnks )
     !
     RETURN
