@@ -53,6 +53,8 @@ MODULE mp_global
   INTEGER :: nproc_bgrp  = 1  ! number of processor within a band group
   INTEGER :: inter_bgrp_comm  = 0  ! inter band group communicator
   INTEGER :: intra_bgrp_comm  = 0  ! intra band group communicator  
+  INTEGER :: ibnd_start = 0 !starting band index
+  INTEGER :: ibnd_end = 0   !ending band index
   !
   ! ... ortho (or linear-algebra) groups
   !
@@ -105,7 +107,7 @@ CONTAINS
     ! ... NPOOL must be a whole divisor of NPROC
     !
     IMPLICIT NONE
-    INTEGER :: world, nproc_ortho_in
+    INTEGER :: world, nproc_ortho_in = 0
     INTEGER :: root = 0
     !
     !
@@ -162,8 +164,8 @@ CONTAINS
        !
        CALL get_arg_northo( nproc_ortho_in )
        !
-       nproc_ortho_in = MAX( nproc_ortho_in, 1 )
-       nproc_ortho_in = MIN( nproc_ortho_in, nproc )
+       ! ... nproc_ortho_in = 0 if no command-line option -ndiag or -northo
+       ! ... nproc_ortho_in = N if -ndiag N or -northo N comm-line opt present
        !
     END IF
     !
@@ -246,7 +248,7 @@ CONTAINS
     IMPLICIT NONE
     !
     INTEGER, INTENT(IN) :: world, root
-    INTEGER :: nproc_ortho_in 
+    INTEGER :: nproc_ortho_in = 0 
     !
     INTEGER :: myrank, npe
     !
@@ -287,9 +289,9 @@ CONTAINS
        !
        CALL get_arg_northo( nproc_ortho_in )
        !
-       nproc_ortho_in = MAX( nproc_ortho_in, 1 )
-       nproc_ortho_in = MIN( nproc_ortho_in, nproc )
-       !
+       ! ... nproc_ortho_in = 0 if no command-line option -ndiag or -northo
+       ! ... nproc_ortho_in = N if -ndiag N or -northo N comm-line opt present
+       ! 
     END IF
     !
     CALL mp_barrier(world)
@@ -385,7 +387,7 @@ CONTAINS
     INTEGER :: parent_nproc = 1
     INTEGER :: parent_mype  = 0
     !
-#if defined (__PARA)
+#if defined (__MPI)
     !
     parent_nproc = mp_size( parent_comm )
     parent_mype  = mp_rank( parent_comm )
@@ -441,7 +443,7 @@ CONTAINS
     INTEGER :: parent_nproc = 1
     INTEGER :: parent_mype  = 0
     !
-#if defined (__PARA)
+#if defined (__MPI)
     !
     parent_nproc = mp_size( parent_comm )
     parent_mype  = mp_rank( parent_comm )
@@ -500,7 +502,7 @@ CONTAINS
     INTEGER :: parent_nproc = 1
     INTEGER :: parent_mype  = 0
     !
-#if defined (__PARA)
+#if defined (__MPI)
     !
     parent_nproc = mp_size( parent_comm )
     parent_mype  = mp_rank( parent_comm )
@@ -543,13 +545,13 @@ CONTAINS
   !
   !----------------------------------------------------------------------------
   SUBROUTINE init_ortho( nproc_ortho_in, parent_comm )
-    !----------------------------------------------------------------------------
+    !---------------------------------------------------------------------------
     !
     ! ... Ortho group initialization
     !
     IMPLICIT NONE
     !
-    INTEGER, INTENT(IN) :: nproc_ortho_in
+    INTEGER, INTENT(IN) :: nproc_ortho_in! read from command-line, 0 if unset
     INTEGER, INTENT(IN) :: parent_comm   ! communicator of the parent group
     !
     INTEGER :: nproc_ortho_try
@@ -568,11 +570,13 @@ CONTAINS
     !
 #endif
     !
-    IF( nproc_ortho_in > 1 ) THEN
-       ! use the command line value ensuring that it falls in the proper range.
+    IF( nproc_ortho_in > 0 ) THEN
+       ! command-line argument -ndiag N or -nproc N set to N
+       ! use the command line value ensuring that it falls in the proper range
        nproc_ortho_try = MIN( nproc_ortho_in , parent_nproc )
-    ELSE
-       ! here we can play with custom architecture specific default definitions
+    ELSE 
+       ! no command-line argument -ndiag N or -nproc N is present
+       ! insert here custom architecture specific default definitions
 #if defined __SCALAPACK
        nproc_ortho_try = MAX( parent_nproc/2, 1 )
 #else
@@ -805,6 +809,41 @@ CONTAINS
      !
   END SUBROUTINE distribute_over_bgrp
   !
+  SUBROUTINE init_index_over_band(comm,nbnd)
+    IMPLICIT NONE
+#if defined (__MPI)
+    !
+    include 'mpif.h'
+    !
+#endif
+    INTEGER, INTENT(IN) :: comm, nbnd
+
+    INTEGER :: npe, myrank, ierror, rest, k
+
+    myrank = mp_rank(comm)
+    npe = mp_size(comm)
+
+!    call mpi_comm_rank(comm, mp_rank, ierror)
+!    call mpi_comm_size(comm, mp_size, ierror)
+    rest = mod(nbnd, npe)
+    k = int(nbnd/npe)
+
+    if(k.ge.1)then
+     if(rest > myrank)then
+       ibnd_start = (myrank)*k + (myrank+1)
+       ibnd_end  =  (myrank+1)*k + (myrank+1)
+     else
+       ibnd_start = (myrank)*k + rest + 1
+       ibnd_end  =  (myrank+1)*k + rest
+     endif
+    else
+     ibnd_start = 1
+     ibnd_end = nbnd
+    endif
+
+
+  END SUBROUTINE init_index_over_band
+  !
   !
   FUNCTION get_ntask_groups()
      IMPLICIT NONE
