@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2002-2009 Quantum ESPRESSO group
+! Copyright (C) 2002-2011 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -10,14 +10,12 @@ MODULE read_cards_module
    !---------------------------------------------------------------------------
    !
    ! ...  This module handles the reading of cards from standard input
-   ! ...  Written by Carlo Cavazzoni and modified for "path" implementation
-   ! ...  by Carlo Sbraccia
+   ! ...  Original version written by Carlo Cavazzoni
    !
    USE kinds,     ONLY : DP
    USE io_global, ONLY : stdout
    USE constants, ONLY : angstrom_au
-   USE parser,    ONLY : field_count, read_line, get_field, &
-                         parse_unit
+   USE parser,    ONLY : field_count, read_line, get_field, parse_unit
    USE io_global, ONLY : ionode, ionode_id
    !
    USE input_parameters
@@ -89,32 +87,14 @@ CONTAINS
       k2       = 0
       k3       = 0
       !
-      ! ... Grids
-      !
-      t2dpegrid_inp = .false.
-      !
       ! ... Electronic states
       !
       tf_inp = .false.
       !
-      ! ... Hartree planar mean
-      !
-      tvhmean_inp = .false.
-      vhnr_inp    = 0
-      vhiunit_inp = 0
-      vhrmin_inp  = 0.0_DP
-      vhrmax_inp  = 0.0_DP
-      vhasse_inp  = 'K'
-      !
       ! ... ion_velocities
       !
       tavel = .false.
       !
-      ! ... setnfi
-      !
-      newnfi_card  = -1
-      tnewnfi_card = .false.
-      !
       CALL init_autopilot()
       !
       RETURN
@@ -161,6 +141,8 @@ CONTAINS
       IF ( trim(card) == 'AUTOPILOT' ) THEN
          !
          CALL card_autopilot( input_line )
+         IF ( prog == 'PW' .and. ionode ) &
+            WRITE( stdout,'(A)') 'Warning: card '//trim(input_line)//' ignored'
          !
       ELSEIF ( trim(card) == 'ATOMIC_SPECIES' ) THEN
          !
@@ -174,36 +156,20 @@ CONTAINS
          !
          CALL card_atomic_forces( input_line, prog )
          !
-      ELSEIF ( trim(card) == 'SETNFI' ) THEN
-         !
-         CALL card_setnfi( input_line )
-         IF ( ( prog == 'PW' .or. prog == 'CP' ) .and. ionode ) &
-            WRITE( stdout,'(A)') 'Warning: card '//trim(input_line)//' ignored'
-         !
       ELSEIF ( trim(card) == 'CONSTRAINTS' ) THEN
          !
          CALL card_constraints( input_line )
          !
-!      ELSEIF ( trim(card) == 'COLLECTIVE_VARS' ) THEN
-         !
-!         CALL card_collective_vars( input_line )
-         !
-      ELSEIF ( trim(card) == 'VHMEAN' ) THEN
-         !
-         CALL card_vhmean( input_line )
-         IF ( ( prog == 'PW' .or. prog == 'CP' ) .and. ionode ) &
-            WRITE( stdout,'(A)') 'Warning: card '//trim(input_line)//' ignored'
-         !
       ELSEIF ( trim(card) == 'DIPOLE' ) THEN
          !
          CALL card_dipole( input_line )
-         IF ( ( prog == 'PW' .or. prog == 'CP' ) .and. ionode ) &
+         IF ( prog == 'PW' .and. ionode ) &
             WRITE( stdout,'(A)') 'Warning: card '//trim(input_line)//' ignored'
          !
       ELSEIF ( trim(card) == 'ESR' ) THEN
          !
          CALL card_esr( input_line )
-         IF ( ( prog == 'PW' .or. prog == 'CP' ) .and. ionode ) &
+         IF ( prog == 'PW' .and. ionode ) &
             WRITE( stdout,'(A)') 'Warning: card '//trim(input_line)//' ignored'
          !
       ELSEIF ( trim(card) == 'K_POINTS' ) THEN
@@ -235,10 +201,6 @@ CONTAINS
          IF ( ( prog == 'PW' ) .and. ionode ) &
             WRITE( stdout,'(a)') 'Warning: card '//trim(input_line)//' ignored'
          !
-!      ELSEIF ( trim(card) == 'CLIMBING_IMAGES' ) THEN
-         !
-!         CALL card_climbing_images( input_line )
-
       ELSEIF ( trim(card) == 'PLOT_WANNIER' ) THEN
          !
          CALL card_plot_wannier( input_line )
@@ -321,14 +283,12 @@ CONTAINS
          !
          CALL read_line( input_line )
          READ( input_line, *, iostat=ierr ) lb_pos, atom_mass(is), psfile
-            CALL errore( ' card_atomic_species ', 'cannot read atomic specie from: '//trim(input_line), abs(ierr))
+            CALL errore( ' card_atomic_species ', &
+                'cannot read atomic specie from: '//trim(input_line), abs(ierr))
          atom_pfile(is) = trim( psfile )
          lb_pos         = adjustl( lb_pos )
          atom_label(is) = trim( lb_pos )
          !
-!             IF ( atom_mass(is) <= 0.0_DP ) THEN
-!                CALL errore( ' card_atomic_species ',' invalid  atom_mass ', is )
-!             END IF
          DO ip = 1, is - 1
             IF ( atom_label(ip) == atom_label(is) ) THEN
                CALL errore( ' card_atomic_species ', &
@@ -383,9 +343,6 @@ CONTAINS
    !    END manual
    !------------------------------------------------------------------------
    !
-   ! ... routine modified for NEB           ( C.S. 21/10/2003 )
-   ! ... routine modified for SMD           ( Y.K. 15/04/2004 )
-   !
    SUBROUTINE card_atomic_positions( input_line, prog )
       !
       USE wrappers, ONLY: feval_infix
@@ -418,11 +375,8 @@ CONTAINS
          CALL errore( 'card_atomic_positions', 'nat out of range', nat )
       ENDIF
       !
-      !
-!new
       CALL allocate_input_ions(ntyp,nat)
       !
-      !
       if_pos = 1
       !
       sp_pos = 0
@@ -443,99 +397,82 @@ CONTAINS
                         & 'unknown option for ATOMIC_POSITION: '&
                         & // input_line, 1 )
          ENDIF
-         IF ( prog == 'FP' ) atomic_positions = 'bohr'
          IF ( prog == 'CP' ) atomic_positions = 'bohr'
          IF ( prog == 'PW' ) atomic_positions = 'alat'
       ENDIF
       !
-
+      reader_loop : DO ia = 1,nat,1
          !
-
-         reader_loop : DO ia = 1,nat,1
-            !
-            CALL read_line( input_line, end_of_file = tend )
-            IF ( tend ) &
-               CALL errore( 'read_cards', &
+         CALL read_line( input_line, end_of_file = tend )
+         IF ( tend ) CALL errore( 'read_cards', &
                            'end of file reading atomic positions', ia )
-            !
-            CALL field_count( nfield, input_line )
-
-            !
-            IF ( sic /= 'none' .and. nfield /= 8 ) &
+         !
+         CALL field_count( nfield, input_line )
+         !
+         IF ( sic /= 'none' .and. nfield /= 8 ) &
                CALL errore( 'read_cards', &
                            'ATOMIC_POSITIONS with sic, 8 columns required', 1 )
-            !
-            IF ( nfield /= 4 .and. nfield /= 7 .and. nfield /= 8) &
+         !
+         IF ( nfield /= 4 .and. nfield /= 7 .and. nfield /= 8) &
                CALL errore( 'read_cards', 'wrong number of columns ' // &
                            & 'in ATOMIC_POSITIONS', ia )
 
-            ! read atom symbol (column 1) and coordinate
-            CALL get_field(1, lb_pos, input_line)
-            lb_pos = trim(lb_pos)
-            !
-            error_msg = 'Error while parsing atomic position card.'
-            ! read field 2 (atom X coordinate)
-            CALL get_field(2, field_str, input_line)
-               rd_pos(1,ia) = feval_infix(ierr, field_str )
-               CALL errore('card_atomic_positions', error_msg, ierr)
-            ! read field 2 (atom Y coordinate)
-            CALL get_field(3, field_str, input_line)
-               rd_pos(2,ia) = feval_infix(ierr, field_str )
-               CALL errore('card_atomic_positions', error_msg, ierr)
-            ! read field 2 (atom Z coordinate)
-            CALL get_field(4, field_str, input_line)
-               rd_pos(3,ia) = feval_infix(ierr, field_str )
-               CALL errore('card_atomic_positions', error_msg, ierr)
-               !
-            IF ( nfield >= 7 ) THEN
-               ! read constrains (fields 5-7, if present)
-               CALL get_field(5, field_str, input_line)
-               READ(field_str, *) if_pos(1,ia)
-               CALL get_field(6, field_str, input_line)
-               READ(field_str, *) if_pos(2,ia)
-               CALL get_field(7, field_str, input_line)
-               READ(field_str, *) if_pos(3,ia)
-            ENDIF
-               !
-            IF ( nfield == 8 ) THEN
-               CALL get_field(5, field_str, input_line)
-               READ(field_str, *) id_loc(ia)
-            ENDIF
-            !
-            match_label: DO is = 1, ntyp
-               !
-               IF ( trim(lb_pos) == trim( atom_label(is) ) ) THEN
-                  !
-                  sp_pos(ia) = is
-                  exit match_label
-                  !
-               ENDIF
-               !
-            ENDDO match_label
-            !
-
-
-            IF( ( sp_pos(ia) < 1 ) .or. ( sp_pos(ia) > ntyp ) ) THEN
-               !
-               CALL errore( 'read_cards', 'species '//trim(lb_pos)// &
-                           & ' in ATOMIC_POSITIONS is nonexistent', ia )
-               !
-            ENDIF
-            !
-            is = sp_pos(ia)
-            !
-            na_inp(is) = na_inp(is) + 1
-            !
-
-         ENDDO reader_loop
+         ! read atom symbol (column 1) and coordinate
+         CALL get_field(1, lb_pos, input_line)
+         lb_pos = trim(lb_pos)
          !
-      !
-!       DO is = 1, ntyp
-!          IF( na_inp( is ) < 1 ) THEN
-!             CALL errore( 'read_cards', &
-!                'no atom found in ATOMIC_POSITIONS for species '//TRIM(atom_label(is)), is )
-!          END IF
-!       END DO
+         error_msg = 'Error while parsing atomic position card.'
+         ! read field 2 (atom X coordinate)
+         CALL get_field(2, field_str, input_line)
+         rd_pos(1,ia) = feval_infix(ierr, field_str )
+         CALL errore('card_atomic_positions', error_msg, ierr)
+         ! read field 2 (atom Y coordinate)
+         CALL get_field(3, field_str, input_line)
+         rd_pos(2,ia) = feval_infix(ierr, field_str )
+         CALL errore('card_atomic_positions', error_msg, ierr)
+         ! read field 2 (atom Z coordinate)
+         CALL get_field(4, field_str, input_line)
+         rd_pos(3,ia) = feval_infix(ierr, field_str )
+         CALL errore('card_atomic_positions', error_msg, ierr)
+               !
+         IF ( nfield >= 7 ) THEN
+            ! read constrains (fields 5-7, if present)
+            CALL get_field(5, field_str, input_line)
+            READ(field_str, *) if_pos(1,ia)
+            CALL get_field(6, field_str, input_line)
+            READ(field_str, *) if_pos(2,ia)
+            CALL get_field(7, field_str, input_line)
+            READ(field_str, *) if_pos(3,ia)
+         ENDIF
+         !
+         IF ( nfield == 8 ) THEN
+            CALL get_field(5, field_str, input_line)
+            READ(field_str, *) id_loc(ia)
+         ENDIF
+         !
+         match_label: DO is = 1, ntyp
+            !
+            IF ( trim(lb_pos) == trim( atom_label(is) ) ) THEN
+               !
+               sp_pos(ia) = is
+               exit match_label
+               !
+            ENDIF
+            !
+         ENDDO match_label
+         !
+         IF( ( sp_pos(ia) < 1 ) .or. ( sp_pos(ia) > ntyp ) ) THEN
+            !
+            CALL errore( 'read_cards', 'species '//trim(lb_pos)// &
+                           & ' in ATOMIC_POSITIONS is nonexistent', ia )
+            !
+         ENDIF
+         !
+         is = sp_pos(ia)
+         !
+         na_inp(is) = na_inp(is) + 1
+         !
+      ENDDO reader_loop
       !
       tapos = .true.
       !
@@ -681,7 +618,7 @@ CONTAINS
       INTEGER            :: nkaux
       INTEGER, ALLOCATABLE :: wkaux(:)
       REAL(DP), ALLOCATABLE :: xkaux(:,:)
-      REAL(DP) :: delta
+      REAL(DP) :: delta, wk0
       LOGICAL, EXTERNAL  :: matches
       LOGICAL            :: tend,terr
       LOGICAL            :: kband = .false.
@@ -725,8 +662,8 @@ CONTAINS
                   ('card_kpoints', 'invalid offsets: must be 0 or 1', 1)
          IF ( nk1 <= 0 .or. nk2 <= 0 .or. nk3 <= 0 ) CALL errore &
                   ('card_kpoints', 'invalid values for nk1, nk2, nk3', 1)
-
-         !
+         ALLOCATE ( xk(3,1), wk(1) ) ! prevents problems with debug flags
+         !                           ! when init_startk is called in iosys
       ELSEIF ( ( k_points == 'tpiba' ) .or. ( k_points == 'crystal' ) ) THEN
          !
          ! ... input k-points are in 2pi/a units
@@ -735,29 +672,46 @@ CONTAINS
          IF (tend) GOTO 10
          IF (terr) GOTO 20
          READ(input_line, *, END=10, ERR=20) nkstot
-         IF ( nkstot > size (xk,2)  ) CALL errore &
-                  ('card_kpoints', 'too many k-points',nkstot)
          !
-         DO i = 1, nkstot
-            CALL read_line( input_line, end_of_file = tend, error = terr )
-            IF (tend) GOTO 10
-            IF (tend) GOTO 20
-            READ(input_line,*, END=10, ERR=20) xk(1,i), xk(2,i), xk(3,i), wk(i)
-         ENDDO
-         IF (kband) THEN
+         IF (.NOT. kband) THEN
+            ALLOCATE ( xk(3, nkstot), wk(nkstot) )
+            DO i = 1, nkstot
+               CALL read_line( input_line, end_of_file = tend, error = terr )
+               IF (tend) GOTO 10
+               IF (terr) GOTO 20
+               READ(input_line,*, END=10, ERR=20) xk(1,i),xk(2,i),xk(3,i),wk(i)
+            ENDDO
+         ELSE
             nkaux=nkstot
-            ALLOCATE(xkaux(3,nkstot))
-            ALLOCATE(wkaux(nkstot))
-            xkaux(:,1:nkstot)=xk(:,1:nkstot)
-            wkaux(1:nkstot)=nint(wk(1:nkstot))
+            ALLOCATE(xkaux(3,nkstot), wkaux(nkstot))
+            DO i = 1, nkstot
+               CALL read_line( input_line, end_of_file = tend, error = terr )
+               IF (tend) GOTO 10
+               IF (terr) GOTO 20
+               READ(input_line,*, END=10, ERR=20) xkaux(1,i), xkaux(2,i), &
+                                                  xkaux(3,i), wk0
+               wkaux(i) = NINT ( wk0 ) ! beware: wkaux is integer
+            ENDDO
+            ! Count k-points first
+            nkstot=0
+            DO i=1,nkaux-1
+               IF ( wkaux(i) > 0 ) THEN
+                  nkstot=nkstot+wkaux(i)
+               ELSEIF ( wkaux(i) == 0 ) THEN
+                  nkstot=nkstot+1
+               ELSE
+                  CALL errore ('card_kpoints', 'wrong number of points',i)
+               ENDIF  
+            ENDDO
+            nkstot=nkstot+1
+            ALLOCATE ( xk(3,nkstot), wk(nkstot) )
+            ! Now fill the points
             nkstot=0
             DO i=1,nkaux-1
                IF (wkaux(i)>0) THEN
                   delta=1.0_DP/wkaux(i)
                   DO j=0,wkaux(i)-1
                      nkstot=nkstot+1
-                     IF ( nkstot > size (xk,2)  ) CALL errore &
-                           ('card_kpoints', 'too many k-points',nkstot)
                      xk(:,nkstot)=xkaux(:,i)+delta*j*(xkaux(:,i+1)-xkaux(:,i))
                      wk(nkstot)=1.0_DP
                   ENDDO
@@ -779,6 +733,7 @@ CONTAINS
       ELSEIF ( k_points == 'gamma' ) THEN
          !
          nkstot = 1
+         ALLOCATE ( xk(3,1), wk(1) )
          xk(:,1) = 0.0_DP
          wk(1) = 1.0_DP
          !
@@ -799,74 +754,6 @@ CONTAINS
    !    BEGIN manual
    !----------------------------------------------------------------------
    !
-   ! SETNFI
-   !
-   !   Reset the step counter to the specified value
-   !
-   ! Syntax:
-   !
-   !  SETNFI
-   !     nfi
-   !
-   ! Example:
-   !
-   !  SETNFI
-   !     100
-   !
-   ! Where:
-   !
-   !    nfi (integer) new value for the step counter
-   !
-   !----------------------------------------------------------------------
-   !    END manual
-   !------------------------------------------------------------------------
-   !
-   SUBROUTINE card_setnfi( input_line )
-      !
-      IMPLICIT NONE
-      !
-      CHARACTER(len=256) :: input_line
-      !
-      !
-      IF ( tsetnfi ) THEN
-         CALL errore( ' card_setnfi ', ' two occurrences', 2 )
-      ENDIF
-      CALL read_line( input_line )
-      READ(input_line,*) newnfi_card
-      tnewnfi_card = .true.
-      tsetnfi = .true.
-      !
-      RETURN
-      !
-   END SUBROUTINE card_setnfi
-   !
-   !
-   !------------------------------------------------------------------------
-   !    BEGIN manual
-   !----------------------------------------------------------------------
-   !
-   ! 2DPROCMESH
-   !
-   !   Distribute the Y and Z FFT dimensions across processors,
-   !   instead of Z dimension only ( default distribution )
-   !
-   ! Syntax:
-   !
-   !    2DPROCMESH
-   !
-   ! Where:
-   !
-   !    no parameters
-   !
-   !----------------------------------------------------------------------
-   !    END manual
-   !------------------------------------------------------------------------
-   !
-   !
-   !------------------------------------------------------------------------
-   !    BEGIN manual
-   !----------------------------------------------------------------------
-   !
    ! OCCUPATIONS
    !
    !   use the specified occupation numbers for electronic states.
@@ -943,7 +830,7 @@ CONTAINS
                !
                f_inp(nbnd_read,is) = feval_infix(ierr, field_str )
                CALL errore('card_occupations',&
-                           'Error parsing occupation: '//trim(field_str), nbnd_read*ierr)
+                  'Error parsing occupation: '//trim(field_str), nbnd_read*ierr)
             ENDDO
          ENDDO
          !
@@ -956,55 +843,6 @@ CONTAINS
       !
    END SUBROUTINE card_occupations
    !
-   !
-   !------------------------------------------------------------------------
-   !    BEGIN manual
-   !----------------------------------------------------------------------
-   !
-   ! VHMEAN
-   !
-   !   Calculation of potential average along a given axis
-   !
-   ! Syntax:
-   !
-   !   VHMEAN
-   !   unit nr rmin rmax asse
-   !
-   ! Example:
-   !
-   !   ????
-   !
-   ! Where:
-   !
-   !   ????
-   !
-   !----------------------------------------------------------------------
-   !    END manual
-   !------------------------------------------------------------------------
-   !
-   SUBROUTINE card_vhmean( input_line )
-      !
-      IMPLICIT NONE
-      !
-      CHARACTER(len=256) :: input_line
-      !
-      !
-      IF ( tvhmean ) THEN
-         CALL errore( ' card_vhmean ', ' two occurrences', 2 )
-      ENDIF
-      !
-      tvhmean_inp = .true.
-      CALL read_line( input_line )
-      READ(input_line,*) &
-         vhiunit_inp, vhnr_inp, vhrmin_inp, vhrmax_inp, vhasse_inp
-      tvhmean = .true.
-      !
-      RETURN
-      !
-   END SUBROUTINE card_vhmean
-   !
-   !
-   !
    !------------------------------------------------------------------------
    !    BEGIN manual
    !----------------------------------------------------------------------
@@ -1138,12 +976,6 @@ CONTAINS
          CALL errore( ' card_cell_parameters ', ' two occurrences', 2 )
       ENDIF
       !
-      IF ( matches( 'HEXAGONAL', input_line ) ) THEN
-         cell_symmetry = 'hexagonal'
-      ELSE
-         cell_symmetry = 'cubic'
-      ENDIF
-      !
       IF ( matches( "BOHR", input_line ) ) THEN
          cell_units = 'bohr'
       ELSEIF ( matches( "ANGSTROM", input_line ) ) THEN
@@ -1346,8 +1178,9 @@ CONTAINS
                                     constr_inp(3,i), &
                                     constr_inp(4,i)
                !
-               WRITE(stdout,'(7x,i3,a,i3,a,i2,a,2f12.6)') &
-                  i,') '//constr_type_inp(i)(1:4),int(constr_inp(1,i)) ,' coordination wrt type:', int(constr_inp(2,i)), &
+               WRITE(stdout,'(7x,i3,a,i3,a,i2,a,2f12.6)') i, &
+                  ') '//constr_type_inp(i)(1:4),int(constr_inp(1,i)) ,&
+                  ' coordination wrt type:', int(constr_inp(2,i)), &
                   ' cutoff distance and smoothing:',  constr_inp(3:4,i)
             ELSEIF ( nfield == 6 ) THEN
                !
@@ -1360,8 +1193,9 @@ CONTAINS
                !
                constr_target_set(i) = .true.
                !
-               WRITE(stdout,'(7x,i3,a,i3,a,i2,a,2f12.6,a,f12.6)') &
-                  i,') '//constr_type_inp(i)(1:4),int(constr_inp(1,i)) ,' coordination wrt type:', int(constr_inp(2,i)), &
+               WRITE(stdout,'(7x,i3,a,i3,a,i2,a,2f12.6,a,f12.6)') i, &
+                  ') '//constr_type_inp(i)(1:4),int(constr_inp(1,i)) , &
+                  ' coordination wrt type:', int(constr_inp(2,i)), &
                   ' cutoff distance and smoothing:',  constr_inp(3:4,i), &
                   '; target:', constr_target_inp(i)
             ELSE
@@ -1390,8 +1224,9 @@ CONTAINS
                !
                constr_target_set(i) = .true.
                !
-               WRITE(stdout,'(7x,i3,a,2i3,a,f12.6)') &
-                  i,') distance between atoms: ', int(constr_inp(1:2,i)), '; target:',  constr_target_inp(i)
+               WRITE(stdout,'(7x,i3,a,2i3,a,f12.6)') i, &
+                  ') distance between atoms: ', int(constr_inp(1:2,i)), &
+                  '; target:',  constr_target_inp(i)
             ELSE
                !
                CALL errore( 'card_constraints', &
@@ -1420,8 +1255,9 @@ CONTAINS
                !
                constr_target_set(i) = .true.
                !
-               WRITE(stdout, '(7x,i3,a,3i3,a,f12.6)') &
-                  i,') planar angle between atoms: ', int(constr_inp(1:3,i)), '; target:', constr_target_inp(i)
+               WRITE(stdout, '(7x,i3,a,3i3,a,f12.6)') i, &
+                  ') planar angle between atoms: ', int(constr_inp(1:3,i)), &
+                  '; target:', constr_target_inp(i)
             ELSE
                !
                CALL errore( 'card_constraints', &
@@ -1452,8 +1288,9 @@ CONTAINS
                !
                constr_target_set(i) = .true.
                !
-               WRITE(stdout, '(7x,i3,a,4i3,a,f12.6)') &
-                  i,') torsional angle between atoms: ', int(constr_inp(1:4,i)), '; target:', constr_target_inp(i)
+               WRITE(stdout, '(7x,i3,a,4i3,a,f12.6)') i, &
+                  ') torsional angle between atoms: ', int(constr_inp(1:4,i)),&
+                  '; target:', constr_target_inp(i)
             ELSE
                !
                CALL errore( 'card_constraints', &
@@ -1471,8 +1308,9 @@ CONTAINS
                                     constr_inp(3,i), &
                                     constr_inp(4,i)
                !
-               WRITE(stdout, '(7x,i3,a,i3,a,3f12.6)') &
-                  i,') bennet projection of atom ', int(constr_inp(1,i)), ' along vector:', constr_inp(2:4,i)
+               WRITE(stdout, '(7x,i3,a,i3,a,3f12.6)') i, &
+                  ') bennet projection of atom ', int(constr_inp(1,i)), &
+                  ' along vector:', constr_inp(2:4,i)
             ELSEIF ( nfield == 6 ) THEN
                !
                READ( input_line, * ) constr_type_inp(i), &
@@ -1484,8 +1322,9 @@ CONTAINS
                !
                constr_target_set(i) = .true.
                !
-               WRITE(stdout, '(7x,i3,a,i3,a,3f12.6,a,f12.6)') &
-                  i,') bennet projection of atom ', int(constr_inp(1,i)), ' along vector:', constr_inp(2:4,i), &
+               WRITE(stdout, '(7x,i3,a,i3,a,3f12.6,a,f12.6)') i, &
+                  ') bennet projection of atom ', int(constr_inp(1,i)), &
+                  ' along vector:', constr_inp(2:4,i), &
                   '; target:', constr_target_inp(i)
             ELSE
                !
@@ -1615,8 +1454,6 @@ CONTAINS
    END SUBROUTINE
    !
    !
-   !
-   !
    !------------------------------------------------------------------------
    !    BEGIN manual
    !----------------------------------------------------------------------
