@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2002-2008 Quantum ESPRESSO group
+! Copyright (C) 2002-2011 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -170,11 +170,13 @@ MODULE read_namelists_module
        nosym = .FALSE.
        nosym_evc = .FALSE.
        force_symmorphic = .FALSE.
+       use_all_frac = .FALSE.
        noinv = .FALSE.
        ecfixed = 0.0_DP
        qcutz   = 0.0_DP
        q2sigma = 0.01_DP
        input_dft = 'none'
+       ecutfock  = -1.0_DP
 !
 ! ... set starting_magnetization to an invalid value:
 ! ... in PW starting_magnetization MUST be set for at least one atomic type
@@ -190,12 +192,14 @@ MODULE read_namelists_module
           !
        END IF
        lda_plus_U = .FALSE.
+       lda_plus_u_kind = 0
        Hubbard_U = 0.0_DP
+       Hubbard_J = 0.0_DP
        Hubbard_alpha = 0.0_DP
-      step_pen=.false.
-      A_pen=0.0_DP
-      sigma_pen=0.01_DP
-      alpha_pen=0.0_DP
+       step_pen=.false.
+       A_pen=0.0_DP
+       sigma_pen=0.01_DP
+       alpha_pen=0.0_DP
        edir = 1
        emaxpos = 0.5_DP
        eopreg = 0.1_DP
@@ -233,10 +237,10 @@ MODULE read_namelists_module
        london_s6   = 0.75_DP
        london_rcut = 200.00_DP
        !
-#ifdef __SOLVENT
-       ! ... Solvent
+#ifdef __ENVIRON
+       ! ... Environ
        !
-       do_solvent = .false. 
+       do_environ = .false. 
        !
 #endif
        ! ... ESM
@@ -252,15 +256,15 @@ MODULE read_namelists_module
        !
      END SUBROUTINE
 
-#ifdef __SOLVENT
+#ifdef __ENVIRON
      !=----------------------------------------------------------------------=!
      !
-     !  Variables initialization for Namelist SOLVENT
+     !  Variables initialization for Namelist ENVIRON
      !
      !=----------------------------------------------------------------------=!
      !
      !-----------------------------------------------------------------------
-     SUBROUTINE solvent_defaults( prog )
+     SUBROUTINE environ_defaults( prog )
        !-----------------------------------------------------------------------
        !
        IMPLICIT NONE
@@ -268,26 +272,38 @@ MODULE read_namelists_module
        CHARACTER(LEN=2) :: prog   ! ... specify the calling program
        !
        !
-       verbose     = 0
-       solvent_thr = 1.D-1
+       verbose      = 0
+       environ_thr  = 1.D-1
+       environ_type = 'input'
        !
        stype   = 1
-       rhozero = 0.001
+       rhomax  = 0.005
        rhomin  = 0.0001
        tbeta   = 4.8
        !
-       epsinfty        = 78.D0
+       env_static_permittivity = 1.D0
        eps_mode        = 'electronic'
        solvationrad(:) = 3.D0
        atomicspread(:) = 0.5D0
+       add_jellium = .false.
        !
+       ifdtype  = 1
+       nfdpoint = 2
+       !
+       mixtype   = 'linear'
+       ndiis     = 1
        mixrhopol = 0.5
        tolrhopol = 1.D-10
        !
-       gamma = 0.D0
+       env_surface_tension = 0.D0
        delta = 0.00001D0
        !
-       extpressure = 0.D0
+       env_pressure = 0.D0
+       !
+       cion = 0.0D0
+       zion = 1.0D0
+       rhopb = 0.0001D0
+       solvent_temperature = 300.0D0
        !
        RETURN
        !
@@ -380,17 +396,6 @@ MODULE read_namelists_module
        fnosee = 1.0_DP
        ampre  = 0.0_DP
        grease = 1.0_DP
-       IF ( prog == 'PW' ) THEN
-          !
-          startingwfc = 'atomic+random'
-          startingpot = 'atomic'
-          !
-       ELSE
-          !
-          startingwfc = 'random'
-          startingpot = ' '
-          !
-       END IF
        conv_thr = 1.E-6_DP
        diis_size = 4
        diis_nreset = 3
@@ -451,11 +456,10 @@ MODULE read_namelists_module
        !
        occupation_constraints = .false.
        !
-#if defined (EXX)
        adaptive_thr   =  .false.
        conv_thr_init  =  0.1E-2_DP
        conv_thr_multi =  0.1_DP
-#endif
+       !
        RETURN
        !
      END SUBROUTINE
@@ -544,22 +548,6 @@ MODULE read_namelists_module
        upscale           = 100.0_DP
        pot_extrapolation = 'atomic'
        wfc_extrapolation = 'none'
-       !
-       !
-       ! ... defaults for "path" optimisations variables
-       !
-!       num_of_images  = 0
-!       first_last_opt = .FALSE.
-!       use_masses     = .FALSE.
-!       use_freezing   = .FALSE.
-!       opt_scheme     = 'quick-min'
-!       temp_req       = 0.0_DP
-!       ds             = 1.0_DP
-!       path_thr       = 0.05_DP
-!       CI_scheme      = 'no-CI'
-!       k_max          = 0.1_DP
-!       k_min          = 0.1_DP
-!       fixed_tan      = .FALSE.
        !
        ! ... BFGS defaults
        !
@@ -704,6 +692,15 @@ MODULE read_namelists_module
        maxwfdt     = 0.30_DP
        wf_q        = 1500.0_DP
        wf_friction = 0.3_DP
+!=======================================================================
+!Lingzhu Kong
+       neigh       = 48
+       vnbsp       = 0
+       poisson_eps = 1.D-6
+       dis_cutoff  = 7.0_DP
+       exx_ps_rcut = 5.0
+       exx_me_rcut = 10.0
+!=======================================================================
        !
        nit    = 10
        nsd    = 10
@@ -827,11 +824,11 @@ MODULE read_namelists_module
        CALL mp_bcast( nosym_evc,         ionode_id )
        CALL mp_bcast( noinv,             ionode_id )
        CALL mp_bcast( force_symmorphic,  ionode_id )
+       CALL mp_bcast( use_all_frac,      ionode_id )
        CALL mp_bcast( ecfixed,           ionode_id )
        CALL mp_bcast( qcutz,             ionode_id )
        CALL mp_bcast( q2sigma,           ionode_id )
        CALL mp_bcast( input_dft,         ionode_id )
-#ifdef EXX
        CALL mp_bcast( nqx1,                   ionode_id )
        CALL mp_bcast( nqx2,                   ionode_id )
        CALL mp_bcast( nqx3,                   ionode_id )
@@ -841,12 +838,14 @@ MODULE read_namelists_module
        CALL mp_bcast( x_gamma_extrapolation,  ionode_id )
        CALL mp_bcast( yukawa,                 ionode_id )
        CALL mp_bcast( ecutvcut,               ionode_id )
-#endif
+       CALL mp_bcast( ecutfock,               ionode_id )
        CALL mp_bcast( starting_magnetization, ionode_id )
        CALL mp_bcast( starting_ns_eigenvalue, ionode_id )
        CALL mp_bcast( U_projection_type,      ionode_id )
        CALL mp_bcast( lda_plus_U,             ionode_id )
+       CALL mp_bcast( lda_plus_u_kind,        ionode_id )
        CALL mp_bcast( Hubbard_U,              ionode_id )
+       CALL mp_bcast( Hubbard_J,              ionode_id )
        CALL mp_bcast( Hubbard_alpha,          ionode_id )
        CALL mp_bcast( step_pen,               ionode_id )
        CALL mp_bcast( A_pen,                  ionode_id )
@@ -880,8 +879,8 @@ MODULE read_namelists_module
        CALL mp_bcast( london_rcut,               ionode_id )
        !
        CALL mp_bcast( no_t_rev,                  ionode_id )
-#ifdef __SOLVENT
-       CALL mp_bcast( do_solvent,                ionode_id )
+#ifdef __ENVIRON
+       CALL mp_bcast( do_environ,                ionode_id )
 #endif
        !
        ! ... ESM method broadcast
@@ -896,15 +895,15 @@ MODULE read_namelists_module
        RETURN
        !
      END SUBROUTINE
-#ifdef __SOLVENT
+#ifdef __ENVIRON
      !=----------------------------------------------------------------------=!
      !
-     !  Broadcast variables values for Namelist SOLVENT
+     !  Broadcast variables values for Namelist ENVIRON
      !
      !=----------------------------------------------------------------------=!
      !
      !-----------------------------------------------------------------------
-     SUBROUTINE solvent_bcast()
+     SUBROUTINE environ_bcast()
        !-----------------------------------------------------------------------
        !
        USE io_global, ONLY : ionode_id
@@ -913,25 +912,37 @@ MODULE read_namelists_module
        IMPLICIT NONE
        !
        CALL mp_bcast( verbose,                    ionode_id )
-       CALL mp_bcast( solvent_thr,                ionode_id )
+       CALL mp_bcast( environ_thr,                ionode_id )
+       CALL mp_bcast( environ_type,               ionode_id )
        !
        CALL mp_bcast( stype,                      ionode_id )
-       CALL mp_bcast( rhozero,                    ionode_id )
+       CALL mp_bcast( rhomax,                     ionode_id )
        CALL mp_bcast( rhomin,                     ionode_id )
        CALL mp_bcast( tbeta,                      ionode_id )
        !
-       CALL mp_bcast( epsinfty,                   ionode_id )
+       CALL mp_bcast( env_static_permittivity,    ionode_id )
        CALL mp_bcast( eps_mode,                   ionode_id )
        CALL mp_bcast( solvationrad,               ionode_id )
        CALL mp_bcast( atomicspread,               ionode_id )
+       CALL mp_bcast( add_jellium,                ionode_id )
        !
+       CALL mp_bcast( ifdtype,                    ionode_id )
+       CALL mp_bcast( nfdpoint,                   ionode_id )
+       !
+       CALL mp_bcast( mixtype,                    ionode_id )
+       CALL mp_bcast( ndiis,                      ionode_id )
        CALL mp_bcast( mixrhopol,                  ionode_id )
        CALL mp_bcast( tolrhopol,                  ionode_id )
        !
-       CALL mp_bcast( gamma,                      ionode_id )
+       CALL mp_bcast( env_surface_tension,        ionode_id )
        CALL mp_bcast( delta,                      ionode_id )
        !
-       CALL mp_bcast( extpressure,                ionode_id )
+       CALL mp_bcast( env_pressure,               ionode_id )
+       !
+       CALL mp_bcast( cion,                       ionode_id )
+       CALL mp_bcast( zion,                       ionode_id )
+       CALL mp_bcast( rhopb,                      ionode_id )
+       CALL mp_bcast( solvent_temperature,        ionode_id )
        !
       RETURN
        !
@@ -1075,11 +1086,9 @@ MODULE read_namelists_module
        !
        ! ... real space ...
        CALL mp_bcast( real_space, ionode_id)
-#if defined (EXX)
        CALL mp_bcast( adaptive_thr,       ionode_id )
        CALL mp_bcast( conv_thr_init,      ionode_id )
        CALL mp_bcast( conv_thr_multi,     ionode_id )
-#endif
        RETURN
        !
      END SUBROUTINE
@@ -1127,21 +1136,6 @@ MODULE read_namelists_module
        CALL mp_bcast( upscale,           ionode_id )
        CALL mp_bcast( pot_extrapolation, ionode_id )
        CALL mp_bcast( wfc_extrapolation, ionode_id )
-       !
-       ! ... "path" variables broadcast
-       !
-!       CALL mp_bcast( num_of_images,      ionode_id )
-!       CALL mp_bcast( first_last_opt,     ionode_id )
-!       CALL mp_bcast( use_masses,         ionode_id )
-!       CALL mp_bcast( use_freezing,       ionode_id )
-!       CALL mp_bcast( fixed_tan,          ionode_id )
-!       CALL mp_bcast( CI_scheme,          ionode_id )
-!       CALL mp_bcast( opt_scheme,         ionode_id )
-!       CALL mp_bcast( temp_req,           ionode_id )
-!       CALL mp_bcast( ds,                 ionode_id )
-!       CALL mp_bcast( k_max,              ionode_id )
-!       CALL mp_bcast( k_min,              ionode_id )
-!       CALL mp_bcast( path_thr,           ionode_id )
        !
        ! ... BFGS
        !
@@ -1164,7 +1158,6 @@ MODULE read_namelists_module
        !
      END SUBROUTINE
      !
-     !
      !=----------------------------------------------------------------------=!
      !
      !  Broadcast variables values for Namelist CELL
@@ -1267,6 +1260,15 @@ MODULE read_namelists_module
        CALL mp_bcast( efz1,        ionode_id )
        CALL mp_bcast( wfsd,        ionode_id )
        CALL mp_bcast( wfdt,        ionode_id )
+!=================================================================
+!Lingzhu Kong
+       CALL mp_bcast( neigh,       ionode_id )
+       CALL mp_bcast( poisson_eps, ionode_id )
+       CALL mp_bcast( dis_cutoff,  ionode_id )
+       CALL mp_bcast( exx_ps_rcut, ionode_id )
+       CALL mp_bcast( exx_me_rcut, ionode_id )
+       CALL mp_bcast( vnbsp,       ionode_id )
+!=================================================================
        CALL mp_bcast( maxwfdt,     ionode_id )
        CALL mp_bcast( wf_q,        ionode_id )
        CALL mp_bcast( wf_friction, ionode_id )
@@ -1390,6 +1392,13 @@ MODULE read_namelists_module
        IF( refg < 0 ) &
          CALL errore( sub_name, ' wrong table interval refg ', 1 )
        !
+#ifdef __LOWMEM
+       IF( wf_collect .EQ. .true. ) &
+         CALL errore( sub_name, ' wf_collect = .true. is not allowed with LOWMEM build ', 1 )
+       IF( prog /= 'CP' ) &
+         CALL errore( sub_name, ' LOWMEM not available in '//prog//' yet ', 1 )
+#endif
+
        RETURN
        !
      END SUBROUTINE
@@ -1408,18 +1417,11 @@ MODULE read_namelists_module
        !
        CHARACTER(LEN=2)  :: prog   ! ... specify the calling program
        CHARACTER(LEN=20) :: sub_name = ' system_checkin '
-#ifdef EXX
        INTEGER           :: i
        LOGICAL           :: allowed
-#endif
        !
        !
-       IF( ibrav < 0 .OR. ibrav > 14 ) THEN
-          IF ( ibrav /= -12 .AND. ibrav /= -5 ) &
-             CALL errore( sub_name ,' ibrav out of range ', MAX( 1, ibrav) )
-       END IF
-       !
-       IF( ( ibrav /= 0 ) .AND. ( celldm(1) == 0.0_DP ) .AND. ( a == 0.0_DP ) ) &
+       IF( ( ibrav /= 0 ) .AND. (celldm(1) == 0.0_DP) .AND. ( a == 0.0_DP ) ) &
            CALL errore( ' iosys ', &
                       & ' invalid lattice parameters ( celldm or a )', 1 )
        !
@@ -1502,8 +1504,6 @@ MODULE read_namelists_module
        !
        ! ... control on EXX variables
        !
-#ifdef EXX       
-       !
        DO i = 1, SIZE( exxdiv_treatment_allowed )
           IF( TRIM(exxdiv_treatment) == exxdiv_treatment_allowed(i) ) allowed = .TRUE.
        END DO
@@ -1520,7 +1520,6 @@ MODULE read_namelists_module
                                           TRIM(exxdiv_treatment) == "vcut_spherical" ) ) &
           CALL errore(sub_name, ' x_gamma_extrapolation cannot be used with vcut', 1 )
        !
-#endif
        RETURN
        !
      END SUBROUTINE
@@ -1614,13 +1613,6 @@ MODULE read_namelists_module
        IF( ion_maxstep < 0 ) &
           CALL errore( sub_name,' ion_maxstep out of range ',1)
        !
-       ! ... general "path" variables checkin
-       !
-       !
-!       full_phs_path_flag = .FALSE.
-!       cg_phs_path_flag   = .FALSE.
-       !
-       !
        IF (sic /= 'none' .and. sic_rloc == 0.0_DP) &
           CALL errore( sub_name, ' invalid sic_rloc with sic activated ', 1 )
        !
@@ -1758,6 +1750,18 @@ MODULE read_namelists_module
              IF ( prog == 'PW' ) &
                 CALL errore( sub_name, ' calculation ' // &
                            & TRIM( calculation ) // ' not implemented ', 1 )
+!=========================================================================
+!Lingzhu Kong
+          CASE ( 'cp-wf-nscf','cp-wf-pbe0','pbe0-nscf' )
+             IF( prog == 'CP' ) THEN
+                occupations       = 'fixed'
+                electron_dynamics = 'damp'
+                ion_dynamics      = 'damp'
+             END IF
+             IF ( prog == 'PW' ) &
+                CALL errore( sub_name, ' calculation ' // &
+                           & TRIM( calculation ) // ' not implemented ', 1 )
+!=========================================================================
           CASE ('relax')
              IF( prog == 'CP' ) THEN
                 electron_dynamics = 'damp'
@@ -1789,27 +1793,7 @@ MODULE read_namelists_module
              ELSE IF( prog == 'PW' ) THEN
                 ion_dynamics = 'beeman'
              END IF
-!          CASE ( 'neb' )
              !
-             ! ... "path" optimizations
-             !
-!             IF( prog == 'CP' ) THEN
-                !
-!                electron_dynamics = 'damp'
-!                ion_dynamics      = 'none'
-!                cell_dynamics     = 'none'
-                !
-!             END IF
-             !
-!          CASE ( 'smd' )
-             !
-!             IF( prog == 'CP' ) THEN
-!                !
-!                electron_dynamics = 'damp'
-!                ion_dynamics      = 'damp'
-!                !
-!             END IF
-!             !
           CASE DEFAULT
              !
              CALL errore( sub_name,' calculation '// &
@@ -1822,11 +1806,11 @@ MODULE read_namelists_module
           IF ( calculation == 'nscf' .OR. calculation == 'bands'  ) THEN
              !
              startingpot = 'file'
-             startingwfc = 'atomic'
+             startingwfc = 'atomic+random'
              !
           ELSE IF ( restart_mode == "from_scratch" ) THEN
              !
-             startingwfc = 'atomic'
+             startingwfc = 'atomic+random'
              startingpot = 'atomic'
              !
           ELSE
@@ -1836,6 +1820,11 @@ MODULE read_namelists_module
              !
           END IF
           !
+       ELSE IF ( prog == 'CP' ) THEN
+          !
+          startingwfc = 'random'
+          startingpot = ' '
+          !
        END IF
        !
        IF ( TRIM( sic ) /= 'none' ) THEN
@@ -1890,7 +1879,7 @@ MODULE read_namelists_module
        !
        IF(PRESENT(unit)) unit_loc = unit
        !
-       IF( prog /= 'PW' .AND. prog /= 'CP' .AND. prog /= 'SM' ) &
+       IF( prog /= 'PW' .AND. prog /= 'CP' ) &
           CALL errore( ' read_namelists ', ' unknown calling program ', 1 )
        !
        ! ... default settings for all namelists
@@ -1901,14 +1890,11 @@ MODULE read_namelists_module
          CALL electrons_defaults( prog )
          CALL ions_defaults( prog )
          CALL cell_defaults( prog )
-#ifdef __SOLVENT
-         CALL solvent_defaults( prog )
+#ifdef __ENVIRON
+         CALL environ_defaults( prog )
 #endif
          CALL ee_defaults( prog )
        ENDIF
-!       IF( prog == 'SM') THEN
-!         CALL path_defaults()
-!       ENDIF
        !
        ! ... Here start reading standard input file
        !
@@ -1978,7 +1964,10 @@ MODULE read_namelists_module
                TRIM( calculation ) == 'cp'       .OR. &
                TRIM( calculation ) == 'vc-cp'    .OR. &
                TRIM( calculation ) == 'smd'      .OR. &
-               TRIM( calculation ) == 'cp-wf') READ( 5, ions, iostat = ios )
+               TRIM( calculation ) == 'cp-wf-nscf' .OR. &   !Lingzhu Kong
+               TRIM( calculation ) == 'cp-wf-pbe0' .OR. &   !Lingzhu Kong
+               TRIM( calculation ) == 'pbe0-nscf'  .OR. &   !Lingzhu Kong
+               TRIM( calculation ) == 'cp-wf' ) READ( 5, ions, iostat = ios )
   
        END IF
        CALL mp_bcast( ios, ionode_id )
@@ -2023,18 +2012,18 @@ MODULE read_namelists_module
        END IF
        !
        CALL press_ai_bcast()
-#ifdef __SOLVENT
+#ifdef __ENVIRON
        !
-       ! ... SOLVENT namelist
+       ! ... ENVIRON namelist
        !
-       IF ( do_solvent ) THEN
+       IF ( do_environ ) THEN
           ios = 0
-          IF( ionode ) READ( 5, solvent, iostat = ios )
+          IF( ionode ) READ( 5, environ, iostat = ios )
           CALL mp_bcast( ios, ionode_id )
           IF( ios /= 0 ) CALL errore( ' read_namelists ', &
-                                    & ' reading namelist solvent ', ABS(ios) )
+                                    & ' reading namelist environ ', ABS(ios) )
        END IF
-       CALL solvent_bcast()
+       CALL environ_bcast()
 #endif
        !
        ! ... EE namelist
@@ -2053,7 +2042,10 @@ MODULE read_namelists_module
        CALL wannier_defaults( prog )
        ios = 0
        IF( ionode ) THEN
-          IF( TRIM( calculation ) == 'cp-wf' ) THEN
+          IF( TRIM( calculation ) == 'cp-wf'       .OR. & ! Lingzhu Kong
+              TRIM( calculation ) == 'cp-wf-nscf'  .OR. & ! Lingzhu Kong
+              TRIM( calculation ) == 'cp-wf-pbe0'  .OR. & ! Lingzhu Kong
+              TRIM( calculation ) == 'pbe0-nscf' ) THEN   ! Lingzhu Kong
              READ( unit_loc, wannier, iostat = ios )
           END IF
        END IF
@@ -2086,10 +2078,6 @@ MODULE read_namelists_module
        !
        ENDIF
        !
-!       IF (prog == 'SM') THEN
-!         CALL path_read_namelist(5)
-!       ENDIF
-       !
        RETURN
        !
      END SUBROUTINE read_namelists
