@@ -7,14 +7,11 @@
 !-------------------------------------------------------------!
 !
 ! cards not yet implemented:
-! SETNFI
 ! KSOUT
 ! AUTOPILOT
 ! ATOMIC_FORCES
-! COLLECTIVE_VARS
 ! PLOT_WANNIER
 ! WANNIER_AC
-! VHMEAN
 ! DIPOLE
 ! ESR
 !
@@ -137,27 +134,10 @@ CONTAINS
        ! ... not yet implemented in xml reading
        iesr_inp = 1
        !
-    CASE ('VHMEAN')
-       ! ... not yet implemented in xml reading
-       tvhmean_inp = .false.
-       vhnr_inp    = 0
-       vhiunit_inp = 0
-       vhrmin_inp  = 0.0_DP
-       vhrmax_inp  = 0.0_DP
-       vhasse_inp  = 'K'
-       !
     CASE ('ION_VELOCITIES')
        ! ... not yet implemented in xml reading
        tavel = .false.
        !
-    CASE ('SETNFI')
-       ! ... not yet implemented in xml reading
-       newnfi_card  = -1
-       tnewnfi_card = .FALSE.
-       !
-       ! ... warning : this variable doesn't exist in any source file
-       t2dpegrid_inp = .FALSE.
-       !
     CASE DEFAULT
        CALL errore ( 'card_default', 'You want to initialize a card that does &
             &not exist or is not yet implemented ( '//trim(card)//' card)', 1 )
@@ -192,7 +172,6 @@ CONTAINS
        !
        !
     CASE ( 'CELL' )
-       CALL mp_bcast( cell_symmetry, ionode_id )
        CALL mp_bcast( ibrav, ionode_id )
        CALL mp_bcast( celldm, ionode_id )
        CALL mp_bcast( A, ionode_id )
@@ -381,20 +360,6 @@ CONTAINS
          &card', ABS( ierr ) )
     !
     IF ( found ) THEN
-       !
-       CALL iotk_scan_attr( attr, 'sym', option, default='cubic', ierr = ierr )
-       IF ( ierr /= 0 ) CALL errore( 'card_xml_cell', 'error scanning sym &
-            &attribute of cell node', abs(ierr) )
-       !
-       IF ( trim(option) == 'hexagonal' ) THEN
-          !
-          cell_symmetry = 'hexagonal'
-          !
-       ELSE
-          !
-          cell_symmetry = 'cubic'
-          !
-       END IF
        !
        CALL iotk_scan_attr( attr, 'type', option, ierr = ierr )
        IF ( ierr /= 0 ) CALL errore( 'card_xml_cell', 'error scanning type &
