@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2005-2008 Quantum ESPRESSO group
+! Copyright (C) 2005-2011 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -49,10 +49,7 @@ MODULE xml_io_base
             write_header, write_control, write_moving_cell,              &
             write_cell, write_ions, write_symmetry, write_planewaves,    &
             write_efield, write_spin, write_magnetization, write_xc,     &
-#ifdef EXX
-            write_exx,                                                   &
-#endif
-            write_occ, write_bz, write_para,    &
+            write_exx, write_occ, write_bz, write_para,                  &
             write_phonon, write_rho_xml, write_wfc, write_eig,           &
             read_wfc, read_rho_xml
   !
@@ -254,7 +251,7 @@ MODULE xml_io_base
       ! ... keep the line below ( this is the old style RESTARTXX ) !!!
       !
       ! dirname = 'RESTART' // int_to_char( runit )
-      ! the next line is to have seperate RESTART for each image
+      ! the next line is to have separate RESTART for each image
       ! KNK_nimage
       ! if (my_image_id > 0) dirname = trim(dirname) // '_' // trim(int_to_char( my_image_id ))
       !
@@ -743,13 +740,11 @@ MODULE xml_io_base
     !
     !
     !------------------------------------------------------------------------
-    SUBROUTINE write_cell( ibrav, symm_type, &
-                           celldm, alat, a1, a2, a3, b1, b2, b3, &
+    SUBROUTINE write_cell( ibrav, celldm, alat, a1, a2, a3, b1, b2, b3, &
                            do_mp, do_mt, do_esm )
       !------------------------------------------------------------------------
       !
       INTEGER,          INTENT(IN) :: ibrav
-      CHARACTER(LEN=*), INTENT(IN) :: symm_type
       REAL(DP),         INTENT(IN) :: celldm(6), alat
       REAL(DP),         INTENT(IN) :: a1(3), a2(3), a3(3)
       REAL(DP),         INTENT(IN) :: b1(3), b2(3), b3(3)
@@ -807,8 +802,6 @@ MODULE xml_io_base
       CALL iotk_write_dat( iunpun, &
                            "BRAVAIS_LATTICE", TRIM( bravais_lattice ) )
       !
-      CALL iotk_write_dat( iunpun, "CELL_SYMMETRY", symm_type )
-      !
       CALL iotk_write_attr( attr, "UNITS", "Bohr", FIRST = .TRUE. )
       CALL iotk_write_dat( iunpun, "LATTICE_PARAMETER", alat, ATTR = attr )
       !
@@ -904,9 +897,7 @@ MODULE xml_io_base
          !
       ENDDO
       !
-      ! BEWARE: the following instruction is part of a ugly hack to allow
-      !         restarting in parallel execution in machines without a
-      !         parallel file system - See read_ions in pw_restart.f90
+      ! This is the original location where PP files are read from
       !
       CALL iotk_write_dat( iunpun, "PSEUDO_DIR", TRIM( pseudo_dir) )
       !
@@ -930,44 +921,12 @@ MODULE xml_io_base
     END SUBROUTINE write_ions
     !
     !------------------------------------------------------------------------
-!   SUBROUTINE write_vdw_df(vdw_table_name, resource_dir, copy_dirname)
-!
-!      CHARACTER(LEN=*), INTENT(IN) :: vdw_table_name
-!      CHARACTER(LEN=*), INTENT(IN) :: resource_dir
-!      CHARACTER(LEN=*), INTENT(IN) :: copy_dirname
-!      !
-!      INTEGER            :: i, flen
-!      CHARACTER(LEN=256) :: file_table
-!  
-!
-!      CALL iotk_write_begin( iunpun, "VDW_KERNEL" )
-!      CALL iotk_write_dat( iunpun, "VDW_KERNEL_NAME", vdw_table_name )
-!      CALL iotk_write_end( iunpun, "VDW_KERNEL" )    
-!
-!      flen = LEN_TRIM( resource_dir )
-!      IF ( resource_dir(flen:flen) /= '/' ) THEN
-!          !
-!          file_table = resource_dir(1:flen) // '/' // vdw_table_name
-!          !
-!      ELSE
-!          !
-!          file_table = resource_dir(1:flen) // vdw_table_name
-!          !
-!      END IF
-!      !
-!      CALL copy_file( TRIM( file_table ), TRIM( copy_dirname ) // "/" // TRIM( vdw_table_name ) )
-!
-!    END SUBROUTINE write_vdw_df
-!
-    !
-    !------------------------------------------------------------------------
-    SUBROUTINE write_symmetry( ibrav, symm_type, nrot, nsym, invsym, noinv, &
+    SUBROUTINE write_symmetry( ibrav, nrot, nsym, invsym, noinv, &
                                time_reversal, no_t_rev, ft, &
                                s, sname, irt, nat, t_rev )
       !------------------------------------------------------------------------
       !
       INTEGER,          INTENT(IN) :: ibrav, nrot, nsym
-      CHARACTER(LEN=*), INTENT(IN) :: symm_type
       LOGICAL,          INTENT(IN) :: invsym, noinv, time_reversal, no_t_rev
       INTEGER,          INTENT(IN) :: s(:,:,:), irt(:,:), nat, t_rev(:)
       REAL(DP),         INTENT(IN) :: ft(:,:)
@@ -978,9 +937,6 @@ MODULE xml_io_base
       !
       CALL iotk_write_begin( iunpun, "SYMMETRIES" )
       !
-      IF ( ibrav == 0 ) &
-         CALL iotk_write_dat( iunpun, "CELL_SYMMETRY", symm_type )
-      !
       CALL iotk_write_dat( iunpun, "NUMBER_OF_SYMMETRIES", nsym )
       CALL iotk_write_dat( iunpun, "NUMBER_OF_BRAVAIS_SYMMETRIES", nrot )
       !
@@ -1007,7 +963,9 @@ MODULE xml_io_base
          !
          CALL iotk_write_dat( iunpun, "ROTATION", s(:,:,i), COLUMNS=3 )
          CALL iotk_write_dat( iunpun, "FRACTIONAL_TRANSLATION", ft(:,i), COLUMNS=3 )
-         CALL iotk_write_dat( iunpun, "EQUIVALENT_IONS", irt(i,1:nat), COLUMNS=8 )
+         !
+         IF ( nat > 0 ) &
+            CALL iotk_write_dat( iunpun, "EQUIVALENT_IONS", irt(i,1:nat), COLUMNS=8 )
          !
          CALL iotk_write_end( iunpun, "SYMM" // TRIM( iotk_index( i ) ) )
          !
@@ -1244,17 +1202,18 @@ MODULE xml_io_base
     END SUBROUTINE write_magnetization
     !
     !------------------------------------------------------------------------
-    SUBROUTINE write_xc( dft, nsp, lda_plus_u, &
-                         Hubbard_lmax, Hubbard_l, Hubbard_U, Hubbard_alpha, &
+    SUBROUTINE write_xc( dft, nsp, lda_plus_u, lda_plus_u_kind, Hubbard_lmax, &
+                         Hubbard_l, Hubbard_U, Hubbard_J, Hubbard_alpha,      &
                          inlc, vdw_table_name, pseudo_dir, dirname )
       !------------------------------------------------------------------------
       !
       CHARACTER(LEN=*),   INTENT(IN) :: dft
       LOGICAL,            INTENT(IN) :: lda_plus_u
+      INTEGER,  OPTIONAL, INTENT(IN) :: lda_plus_u_kind
       INTEGER,  OPTIONAL, INTENT(IN) :: nsp
       INTEGER,  OPTIONAL, INTENT(IN) :: Hubbard_lmax
       INTEGER,  OPTIONAL, INTENT(IN) :: Hubbard_l(:)
-      REAL(DP), OPTIONAL, INTENT(IN) :: Hubbard_U(:), Hubbard_alpha(:)
+      REAL(DP), OPTIONAL, INTENT(IN) :: Hubbard_U(:), Hubbard_J(:,:), Hubbard_alpha(:)
       INTEGER,  OPTIONAL, INTENT(IN) :: inlc
       CHARACTER(LEN=*), OPTIONAL,   INTENT(IN) :: vdw_table_name, pseudo_dir, dirname
       !
@@ -1272,6 +1231,7 @@ MODULE xml_io_base
          IF ( .NOT. PRESENT( Hubbard_lmax ) .OR. &
               .NOT. PRESENT( Hubbard_l )    .OR. & 
               .NOT. PRESENT( Hubbard_U )    .OR. &
+              .NOT. PRESENT( Hubbard_J )    .OR. &
               .NOT. PRESENT( nsp )          .OR. &
               .NOT. PRESENT( Hubbard_alpha ) ) &
             CALL errore( 'write_exchange_correlation', &
@@ -1279,6 +1239,8 @@ MODULE xml_io_base
          !
          CALL iotk_write_dat( iunpun, "NUMBER_OF_SPECIES", nsp )
          !
+         CALL iotk_write_dat( iunpun, "LDA_PLUS_U_KIND", lda_plus_u_kind )
+         ! 
          CALL iotk_write_dat( iunpun, "HUBBARD_LMAX", Hubbard_lmax )
          !
          CALL iotk_write_dat( iunpun, "HUBBARD_L", &
@@ -1286,6 +1248,8 @@ MODULE xml_io_base
          !
          CALL iotk_write_dat( iunpun, "HUBBARD_U", Hubbard_U(1:nsp) )
          !
+         CALL iotk_write_dat( iunpun, "HUBBARD_J", Hubbard_J(1:3,1:nsp), COLUMNS = 3 )
+         !
          CALL iotk_write_dat( iunpun, "HUBBARD_ALPHA", Hubbard_alpha(1:nsp) )
          !
       END IF
@@ -1328,14 +1292,13 @@ MODULE xml_io_base
       !
     END SUBROUTINE write_xc
 
-#ifdef EXX
     !------------------------------------------------------------------------
     SUBROUTINE write_exx( x_gamma_extrapolation, nqx1, nqx2, nqx3, &
                           exxdiv_treatment, yukawa, ecutvcut, exx_fraction, &
-                          screening_parameter )
+                          screening_parameter, exx_is_active )
       !------------------------------------------------------------------------
       !
-      LOGICAL,            INTENT(IN) :: x_gamma_extrapolation
+      LOGICAL,            INTENT(IN) :: x_gamma_extrapolation, exx_is_active
       INTEGER,  OPTIONAL, INTENT(IN) :: nqx1, nqx2, nqx3
       CHARACTER(LEN=*),   INTENT(IN) :: exxdiv_treatment
       REAL(DP),           INTENT(IN) :: yukawa, ecutvcut, exx_fraction
@@ -1351,9 +1314,9 @@ MODULE xml_io_base
       call iotk_write_dat(iunpun, "ecutvcut", ecutvcut)
       call iotk_write_dat(iunpun, "exx_fraction", exx_fraction)
       call iotk_write_dat(iunpun, "screening_parameter", screening_parameter)
+      call iotk_write_dat(iunpun, "exx_is_active", exx_is_active)
       CALL iotk_write_end(iunpun, "EXACT_EXCHANGE" )
     END SUBROUTINE write_exx
-#endif
     !
     !------------------------------------------------------------------------
     SUBROUTINE write_occ( lgauss, ngauss, degauss, ltetra, ntetra, &
@@ -1465,6 +1428,7 @@ MODULE xml_io_base
       ! ... these are k-points and weights in the Irreducible BZ
       !
       IF (present(nks_start).and.present(xk_start).and.present(wk_start)) THEN
+         !
          CALL iotk_write_dat( iunpun, "STARTING_K-POINTS", nks_start )
          !
          DO ik = 1, nks_start
@@ -1827,13 +1791,15 @@ MODULE xml_io_base
     ! ... methods to write and read wavefunctions
     !
     !------------------------------------------------------------------------
-    SUBROUTINE write_wfc( iuni, ik, nk, kunit, ispin, &
-                          nspin, wf0, ngw, gamma_only, nbnd, igl, ngwl, filename, scalef, &
-                          ionode, root_in_group, intra_group_comm, inter_group_comm, parent_group_comm )
+    SUBROUTINE write_wfc( iuni, ik, nk, kunit, ispin, nspin, wf0, ngw,   &
+                          gamma_only, nbnd, igl, ngwl, filename, scalef, &
+                          ionode, root_in_group, intra_group_comm,       &
+                          inter_group_comm, parent_group_comm )
       !------------------------------------------------------------------------
       !
       USE mp_wave,    ONLY : mergewf
       USE mp,         ONLY : mp_get, mp_size, mp_rank, mp_sum
+      USE control_flags,     ONLY : lwfnscf, lwfpbe0nscf  ! Lingzhu Kong
       !
       IMPLICIT NONE
       !
@@ -1895,7 +1861,11 @@ MODULE xml_io_base
       ALLOCATE( wtmp( MAX( igwx, 1 ) ) )
       !
       wtmp = 0.0_DP
-      !
+      ! Next 3 lines: Lingzhu Kong
+      IF ( ( index(filename,'evc0') > 0 ) .and. (lwfnscf .or. lwfpbe0nscf) )THEN
+         IF ( ionode ) OPEN(60,file='cp_wf.dat',status='unknown',form='unformatted')
+      ENDIF
+
       DO j = 1, nbnd
          !
          IF ( ngroup > 1 ) THEN
@@ -1917,9 +1887,18 @@ MODULE xml_io_base
          !
          IF ( ionode ) &
             CALL iotk_write_dat( iuni, "evc" // iotk_index( j ), wtmp(1:igwx) )
+         ! Next 3 lines : Lingzhu Kong
+         IF ( ( index(filename,'evc0') > 0 ) .and. (lwfnscf .or. lwfpbe0nscf) ) THEN
+            IF ( ionode ) write(60)wtmp(1:igwx) 
+         ENDIF
          !
       END DO
-      !
+      ! Next 4 lines : Lingzhu Kong
+      IF ( ( index(filename,'evc0') > 0 ) .and. (lwfnscf .or. lwfpbe0nscf) )THEN
+          IF ( ionode ) close(60)   !Lingzhu Kong
+          write(*,*)'done writing evc0'
+      ENDIF
+
       IF ( ionode ) CALL iotk_close_write( iuni )
       !
       DEALLOCATE( wtmp )
@@ -2033,6 +2012,10 @@ MODULE xml_io_base
                                    "evc" // iotk_index( j ), wtmp(1:igwx_) )
                !
                IF ( igwx > igwx_ ) wtmp((igwx_+1):igwx) = 0.0_DP
+               ! ===========================================================
+               !       Lingzhu Kong
+               !IF ( j .eq. 1)write(*,'(10f12.5)')(wtmp(i),i=1,igwx_)
+               ! ===========================================================
                !
             END IF
             !
