@@ -1211,10 +1211,10 @@ end function dpz
       V2XA=0.0D0
       V1XB=0.0D0
       V2XB=0.0D0
-      IF(ABS(RHOA).GT.SMALL) THEN
+      IF(RHOA.GT.SMALL.AND.GRHOAA.GT.SMALL) THEN
         CALL pbexsr(2.D0*RHOA, 4.D0*GRHOAA, SXA, V1XA, V2XA, OMEGA)
       ENDIF
-      IF(ABS(RHOB).GT.SMALL) THEN
+      IF(RHOB.GT.SMALL.AND.GRHOBB.GT.SMALL) THEN
         CALL pbexsr(2.D0*RHOB, 4.D0*GRHOBB, SXB, V1XB, V2XB, OMEGA)
       ENDIF
       sx = 0.5D0*(SXA+SXB)
